import pydash
import asyncio
from PIL import Image
import os
import json


class MapManager():
    def __init__(self, context):
        self._context = context
        self._grid_map_images = {}
        self._map_path_images = {}
        self._prohibit_zones = {}
        self._map_list = None

        self._working_dir = context.worker._working_root

        try:
            feature = asyncio.run_coroutine_threadsafe(self.download_maps(), context.event_loop)
            self._map_list = feature.result()
        except Exception as err:
            self._context.rocon_logger.error('Exception occurred', exception=err)
            err.with_traceback()

        self._context.rocon_logger.debug('downloaded maps successfully')

    async def download_maps(self):
        try:
            self._map_list = await self._context.api_configuration.get_maps()

            def cb(m):
                return pydash.get(m, 'site')

            self._map_list = pydash.filter_(self._map_list, cb)
            if len(self._map_list) is 0:
                self._context.rocon_logger.debug('there are no maps on site configuration')
                self._map_list = None
                return self._map_list

            self._grid_map_images = {}
            self._map_path_images = {}

            path = self._working_dir + '/assets/maps/'

            for map in self._map_list:
                map_id = map['id']
                self._grid_map_images[map_id] = await self.get_map_image_from_configuration(map)

                if not os.path.exists(path):
                    os.makedirs(path)

                # to save image
                image_file_name = path + '{}.bmp'.format(map_id)
                self._grid_map_images[map_id].save(image_file_name)

                json_file_name = path + '{}.json'.format(map_id)
                with open(json_file_name, 'w') as outfile:
                    json.dump(map, outfile)

                self._map_path_images[map_id] = Image.open(image_file_name)

                # for test
                #self._map_path_images[map_id].show()
                self._prohibit_zones[map_id] = await self._context.api_configuration.get_zones('prohibit', map_id)

        except Exception as err:
            self._context.rocon_logger.error('failed to download maps')
            self._map_list = None

        return self._map_list

    def get_map_list(self):
        return self._map_list

    def get_map_image(self, map_id) -> Image:
        if map_id not in self._grid_map_images:
            self._context.rocon_logger.debug('not found map image for map id as {}'.format(map_id))
            return None

        return self._grid_map_images[map_id]

    def get_map_image_by_name(self, name) -> Image:

        for map in self._map_list:
            if map['name'] == name:
                map_id = map['id']
                return self._grid_map_images[map_id]

        return None

    def get_map_path_image_by_name(self, name) -> Image:
        for map in self._map_list:
            if map['name'] == name:
                map_id = map['id']
                return self._map_path_images[map_id]
        return None

    def clear_map_path_image_by_name(self, name):
        org_image = self.get_map_image_by_name(name)
        if org_image:
            path_image = self.get_map_path_image_by_name(name)

            path_image.paste(org_image)

    def get_map_images(self):
        return self._grid_map_images

    def get_prohibit_zones(self, map_id):
        if map_id not in self._prohibit_zones:
            self._context.rocon_logger.debug('not found prohibit zone for map id as {}'.format(map_id))
            return None

        return self._prohibit_zones[map_id]

    def get_prohibit_zones_list(self):
        return self._prohibit_zones

    async def get_map_image_from_configuration(self, map)-> Image:

        grid_map_image = None

        try:
            grid_map_image = await self._context.api_configuration.get_map_image('/public/' + pydash.get(map, 'files.monochrome'))

            #grid_map_image.show()

        except Exception as err:
            self._context.rocon_logger.error('Exception occurred', exception=err)
            err.with_traceback()
            return None

        return grid_map_image

    def get_map_names(self):
        names = pydash.map_(self._map_list, lambda map: map['name'])

        return names