import py_trees
import asyncio
import pydash
from overrides import overrides
from .task import Task, AsyncTask
from rocon_client_sdk_py.const import *


class TaskHandleControlMessage(AsyncTask):
    def __init__(self, name="handleControlMessage"):
        super(TaskHandleControlMessage, self).__init__(name)

    @overrides
    def setup(self):
        pass

    @overrides
    def initialise(self):
        self.rocon_logger.debug("initialise >>", module_keyword=BT_KEYWORD)

        #TODO messages가 1개 이상일때, 현재 순차 처리 방식인데 개선 필요.
        messages = self.context.blackboard.get('recv_messages')
        if messages and len(messages) > 0:

            self.async_task_status = py_trees.common.Status.RUNNING

            try:
                asyncio.run_coroutine_threadsafe(self._do_work(), self.context.event_loop)
            except Exception as err:
                self.rocon_logger.error('Exception occurred', module_keyword=BT_KEYWORD, exception=err)
                err.with_traceback()

        else:
            self.async_task_status = py_trees.common.Status.SUCCESS



    async def _do_work(self):

        message_manager = self.context.message_manager
        messages = self.context.blackboard.get('recv_messages')
        if messages and len(messages) > 0:

            msg_idx = pydash.find_index(messages, {'receiver': self.context.worker.uuid})
            if msg_idx != -1:
                msg = pydash.clone(messages[msg_idx])
                self.rocon_logger.debug('start handle message :{}'.format(msg), module_keyword=BT_KEYWORD)
                pydash.pull_at(messages, msg_idx)

                self.context.blackboard.set('recv_messages', messages)
                message_inst = message_manager.find_message(msg['name'])
                if message_inst is None:
                    self.rocon_logger.debug('malformed message handler: {}, {}'.format(msg['key'], {'message': msg}), module_keyword=BT_KEYWORD)
                    self.async_task_status = py_trees.common.Status.FAILURE
                    return

                try:
                    message_body = msg['body']
                    self.rocon_logger.debug('=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=', show_caller_info=False)
                    self.rocon_logger.debug(' worker >> on_message                    ', show_caller_info=False)
                    self.rocon_logger.debug('=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=', show_caller_info=False)
                    self.rocon_logger.debug(' message name : {}, body : {}'.format(message_inst.name, message_body))

                    await self.context.worker.on_message(self.context, message_inst, message_body)

                except Exception as err:
                    self.rocon_logger.error('error occurred while processing message : {}'.format({
                        'message':msg
                    }), module_keyword=BT_KEYWORD, exception=err)

        self.async_task_status = py_trees.common.Status.SUCCESS


    @overrides
    def update(self):
        self.rocon_logger.debug('update >> {}'.format(self.async_task_status), module_keyword=BT_KEYWORD)
        return self.async_task_status

    @overrides
    def terminate(self, new_status):
        self.rocon_logger.debug("terminate >>", module_keyword=BT_KEYWORD)

