import logging
import inspect
from rocon_client_sdk_py.utils.util import *

DEFAULT_SHOW_CALLER_INFO = True
DEFAULT_SAVE_TO_SERVER = False


class RoconLogger(metaclass=SingletonMetaClass):
    def __init__(self):
        self._log_filename = 'default.log'
        self.show_caller_info = DEFAULT_SHOW_CALLER_INFO
        self.enable = True

        self._logging_level = logging.DEBUG

        self._logger = logging.getLogger('RoconLogger')
        self._logger.setLevel(self._logging_level)

        self._console_hangler = logging.StreamHandler()
        self._console_hangler.setLevel(self._logging_level)

        formatter = logging.Formatter('%(asctime)s - [%(levelname)s] - %(message)s')
        self._console_hangler.setFormatter(formatter)
        self._logger.addHandler(self._console_hangler)

        self._disabled_logging_module_keywords = set()

    def debug(self, log_msg='', module_keyword = '', show_caller_info=DEFAULT_SHOW_CALLER_INFO, save_to_server=DEFAULT_SAVE_TO_SERVER, exception=None):

        if module_keyword in self._disabled_logging_module_keywords:
            return

        caller_info = self._get_caller_info(inspect.currentframe().f_back, show_caller_info)
        mod_keyword = '[{}] '.format(module_keyword) if len(module_keyword) > 0 else ''
        text = '{}{}{}'.format(mod_keyword, caller_info, log_msg)
        self._logger.debug(text)

    def info(self, log_msg='', module_keyword = '', show_caller_info=DEFAULT_SHOW_CALLER_INFO, save_to_server=DEFAULT_SAVE_TO_SERVER, exception=None):
        caller_info = self._get_caller_info(inspect.currentframe().f_back, show_caller_info)
        mod_keyword = '[{}] '.format(module_keyword) if len(module_keyword) > 0 else ''
        text = '{}{}{}'.format(mod_keyword, caller_info, log_msg)
        self._logger.info(text)

    def cli_info(self, log_msg='', module_keyword = '', show_caller_info=DEFAULT_SHOW_CALLER_INFO, save_to_server=DEFAULT_SAVE_TO_SERVER, exception=None):
        caller_info = self._get_caller_info(inspect.currentframe().f_back, show_caller_info)
        mod_keyword = '[{}] '.format(module_keyword) if len(module_keyword) > 0 else ''
        text = '{}{}{}'.format(mod_keyword, caller_info, log_msg)
        print(text)

    def warning(self, log_msg='', module_keyword = '', show_caller_info=DEFAULT_SHOW_CALLER_INFO, save_to_server=DEFAULT_SAVE_TO_SERVER, exception=None):
        caller_info = self._get_caller_info(inspect.currentframe().f_back, show_caller_info)
        mod_keyword = '[{}] '.format(module_keyword) if len(module_keyword) > 0 else ''
        text = '{}{}{}'.format(mod_keyword, caller_info, log_msg)
        self._logger.warning(text)

    def error(self, log_msg='', module_keyword = '', show_caller_info=DEFAULT_SHOW_CALLER_INFO, save_to_server=DEFAULT_SAVE_TO_SERVER, exception=None):
        caller_info = self._get_caller_info(inspect.currentframe().f_back, show_caller_info)
        mod_keyword = '[{}] '.format(module_keyword) if len(module_keyword) > 0 else ''
        exc_msg = '\n {}'.format(exception) if exception is not None else ''

        text = '{}{}{}{}'.format(mod_keyword, caller_info, log_msg, exc_msg)
        self._logger.error(text)

    def critical(self, log_msg='', module_keyword = '', show_caller_info=DEFAULT_SHOW_CALLER_INFO, save_to_server=DEFAULT_SAVE_TO_SERVER, exception=None):
        caller_info = self._get_caller_info(inspect.currentframe().f_back, show_caller_info)

        mod_keyword = '[{}] '.format(module_keyword) if len(module_keyword) > 0 else ''
        exc_msg = '\n {}'.format(exception) if exception is not None else ''

        text = '{}{}{}{}'.format(mod_keyword, caller_info, log_msg, exc_msg)
        self._logger.critical(text)

    def _get_caller_info(self, frame, show_caller_info=DEFAULT_SHOW_CALLER_INFO, save_to_server=DEFAULT_SAVE_TO_SERVER, exception=None):
        if show_caller_info is False or self.show_caller_info is False:
            return ''

        if 'self' in frame.f_locals:
            # caller is method of class
            caller_class_name = frame.f_locals["self"].__class__.__name__
            caller_method_name = frame.f_code.co_name

            caller_info = '{}.{}() : '.format(caller_class_name, caller_method_name)
        else:
            caller_method_name = frame.f_code.co_name
            caller_info = '{}() : '.format(caller_method_name)


        return caller_info

    def disable_logging_by_moduel_keyword(self, keyword):
        self._disabled_logging_module_keywords.add(keyword)
        #print(self._disabled_logging_module_keywords)

    def enable_logging_by_moduel_keyword(self, keyword):
        self._disabled_logging_module_keywords.remove(keyword)


rocon_logger = RoconLogger()
