from PIL import Image
import io
import json
from .api import Api


class ApiConfiguration(Api):

    @property
    def request(self):
        return self._httpclient.request()

    @property
    def hostname(self):
        return self._httpclient.hostname_config

    def get_url(self, sub_url):
        url = self._httpclient.config_url(sub_url)
        return url

    async def get_stations(self, station_id=''):
        request = self._httpclient.request()

        sub_id = '/' + station_id if station_id != '' else ''
        url = self._httpclient.config_url('/api/stations' + sub_id)
        self.rocon_logger.debug('request.get > {}'.format(url))

        try:
            async with request.get(url) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error('Exception occurred', exception=exc)
            return None

    async def get_locations(self, destination_id=''):
        request = self._httpclient.request()

        if len(destination_id) > 0:
            url = self._httpclient.config_url('/api/locations/' + destination_id)
        else:
            url = self._httpclient.config_url('/api/locations')

        self.rocon_logger.debug('request.get > {}'.format(url))
        try:
            async with request.get(url) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error('Exception occurred', exception=exc)
            return None

    async def get_resource(self, resource_id):
        request = self._httpclient.request()

        url = self._httpclient.config_url('/api/resources/' + resource_id)
        self.rocon_logger.debug('request.get > {}'.format(url))

        try:
            async with request.get(url) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error('Exception occurred', exception=exc)
            return None

    async def get_resources(self):
        request = self._httpclient.request()

        url = self._httpclient.config_url('/api/resources')
        self.rocon_logger.debug('request.get > {}'.format(url))

        try:
            async with request.get(url) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error('Exception occurred', exception=exc)
            return None

    async def get_maps(self, map=''):
        request = self._httpclient.request()
        map_sub = '/' + map if len(map) > 0 else ''
        url = self._httpclient.config_url('/api/maps' + map_sub)
        self.rocon_logger.debug('request.get > {}'.format(url))

        try:
            async with request.get(url) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error('Exception occurred', exception=exc)
            return None

    async def get_map_image(self, path) -> Image:
        request = self._httpclient.request()
        url = self._httpclient.config_url(path)
        self.rocon_logger.debug('request.get > {}'.format(url))

        try:
            async with request.get(url) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                data = await r.read()
                f = io.BytesIO(data)
                image = Image.open(f)

            if image is None:
                self.rocon_logger.error('image is none')

            return image

        except Exception as exc:
            self.rocon_logger.error('Exception occurred', exception=exc)
            exc.with_traceback()
            return None


    async def get_zones(self, zone_type, map_id):
        query = {'map':map_id, 'type':zone_type}
        request = self._httpclient.request()
        url = self._httpclient.config_url('/api/zones')
        self.rocon_logger.debug('request.get > {}'.format(url))

        try:
            async with request.get(url, params=query) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error('Exception occurred', exception=exc)
            return None

    async def return_resource(self, worker_id, resource_id, slot_id = None):
        request = self._httpclient.request()
        url = self._httpclient.config_url('/api/resources/{}/slots/{}'.format(resource_id, slot_id))
        req_json_body = json.dumps({'status': 'gone'})

        try:
            async with request.put(url, data=req_json_body) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error('Exception occurred', exception=exc)
            return None


    async def get_teleporter_gate(self, teleporter_id, map_id):
        request = self._httpclient.request()
        url = self._httpclient.config_url('/api/teleporter-gates')
        query = {
            'teleporter': teleporter_id,
            'map': map_id,
            'populate':'resource_slots'
        }

        self.rocon_logger.debug('request.get > {}'.format(url))

        try:
            async with request.get(url, params=query) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data[0]

        except Exception as exc:
            self.rocon_logger.error('Exception occurred', exception=exc)
            return None