import json
from .api import Api
from rocon_client_sdk_py.const import *


class ApiReport(Api):

    @property
    def hostname(self):
        return self._httpclient.hostname_concert

    async def req_recommend(self, worker_id):
        request = self._httpclient.request()
        url = self._httpclient.scheduler_url('/reports/recommend')
        self.rocon_logger.debug('request.post > {}'.format(url), module_keyword=BT_KEYWORD)

        try:
            async with request.post(url, json={'worker': worker_id}) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status), module_keyword=BT_KEYWORD)
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data), module_keyword=BT_KEYWORD)
                return json_data

        except Exception as exc:
            self.rocon_logger.error(exc)
            return None

    async def req_ownership(self, report_id, worker_id):
        request = self._httpclient.request()

        req_json_body = json.dumps({'worker': worker_id})
        url = self._httpclient.scheduler_url('/reports/' + report_id + '/ownership')
        self.rocon_logger.debug('request.put > {}'.format(url))

        try:
            async with request.put(url, data=req_json_body) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error(exc)
            return None

    async def update_report(self, report_id, update_body):
        request = self._httpclient.request()

        req_json_body = json.dumps(update_body)
        url = self._httpclient.scheduler_url('/reports/' + report_id)
        self.rocon_logger.debug('request.put > {}'.format(url), module_keyword=BT_KEYWORD)

        try:
            async with request.put(url, data=req_json_body) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status), module_keyword=BT_KEYWORD)
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data), module_keyword=BT_KEYWORD)
                return json_data

        except Exception as exc:
            self.rocon_logger.error(exc)
            return None

    async def get_reports(self, options):
        request = self._httpclient.request()

        url = self._httpclient.scheduler_url('/reports')
        self.rocon_logger.debug('request.get > {}'.format(url), module_keyword=BT_KEYWORD)

        try:
            async with request.get(url, params=options) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status), module_keyword=BT_KEYWORD)
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data), module_keyword=BT_KEYWORD)
                return json_data

        except Exception as exc:
            self.rocon_logger.error(exc)
            return None

    async def get_report_by_id(self, report_id):
        request = self._httpclient.request()

        url = self._httpclient.scheduler_url('/reports/' + report_id)
        self.rocon_logger.debug('request.get > {}'.format(url), module_keyword=BT_KEYWORD)

        try:
            async with request.get(url) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status), module_keyword=BT_KEYWORD)
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data), module_keyword=BT_KEYWORD)
                return json_data

        except Exception as exc:

            if r.status is 404:
                return None
            else:
                self.rocon_logger.error('unhandled error on get_report_by_id')

            self.rocon_logger.error(exc)
            return None

    async def cancel_report(self, report_id, message, force=False):
        request = self._httpclient.request()

        req_json_body = json.dumps({'message': message, 'force': force})
        url = self._httpclient.scheduler_url('/reports/' + report_id + '/cancel')
        self.rocon_logger.debug('request.put > {}'.format(url))

        try:
            async with request.put(url, data=req_json_body) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error(exc)
            return None

    async def abort_report(self, report_id, message):
        request = self._httpclient.request()
        req_json_body = json.dumps({'message': message})
        url = self._httpclient.scheduler_url('/reports/' + report_id + '/abort')
        self.rocon_logger.debug('request.put > {}'.format(url))

        try:
            async with request.put(url, data=req_json_body) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error(exc)
            return None

    async def handle_revision(self, report, propositions, action):
        request = self._httpclient.request()

        report_body = {'action': action, 'propositions': propositions}
        req_json_body = json.dumps(report_body)
        rep = report['id'] if 'id' in report else report
        url = self._httpclient.scheduler_url('/reports/' + rep + '/revision')
        self.rocon_logger.debug('request.put > {}'.format(url))

        try:
            async with request.put(url, data=req_json_body) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status))
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data))
                return json_data

        except Exception as exc:
            self.rocon_logger.error(exc)
            return None

    async def approve_revision(self, report):
        return await self.handle_revision(report['id'], report['revision']['propositions'], 'approve')

    async def reject_revision(self, report):
        return await self.handle_revision(report['id'], report['revision']['propositions'], 'reject')