import pydash
import json
from .api import Api
from rocon_client_sdk_py.const import *


class ApiWorker(Api):

    @property
    def hostname(self):
        return self._httpclient.hostname_concert

    async def find_one_by_uuid(self, uuid):
        request = self._httpclient.request()
        url = self._httpclient.scheduler_url('/workers?uuid=' + uuid)
        self.rocon_logger.debug('request.get > {}'.format(url), module_keyword=BT_KEYWORD)

        async with request.get(url) as r:
            self.rocon_logger.debug('response status > {}'.format(r.status), module_keyword=BT_KEYWORD)
            #TODO error
            json_data = await r.json()
            self.rocon_logger.debug('response data > \n{}'.format(json_data), module_keyword=BT_KEYWORD)
            if len(json_data) > 0:
                return json_data[0]

            self.rocon_logger.error('Not registered : ' + uuid)
            return {}

    async def upsert(self, body):
        request = self._httpclient.request()
        req_body = pydash.pick(body, ['status', 'name', 'type_specific', 'release_version', 'type', 'uuid'])
        req_json_body = json.dumps(req_body)

        url = self._httpclient.scheduler_url('/workers/')
        self.rocon_logger.debug('request.put > {}'.format(url), module_keyword=BT_KEYWORD)
        try:

            headers = {'Content-type': 'application/json'}
            async with request.put(url, data=req_json_body) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status), module_keyword=BT_KEYWORD)

                #TODO error
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data), module_keyword=BT_KEYWORD)
                return json_data

        except Exception as exc:

            self.rocon_logger.error(exc)
            pass


    async def update_worker(self, worker_id, update):
        allowed_update = ['type_specific', 'status', 'name']
        update_body = pydash.pick(update, allowed_update)
        request = self._httpclient.request()
        req_json_body = json.dumps(update_body)
        url = self._httpclient.scheduler_url('/workers/' + worker_id)
        self.rocon_logger.debug('request.put > {}'.format(url), module_keyword=BT_KEYWORD)
        try:
            headers = {'Content-type': 'application/json'}
            async with request.put(url, data=req_json_body) as r:
                self.rocon_logger.debug('response status > {}'.format(r.status), module_keyword=BT_KEYWORD)
                # TODO error
                json_data = await r.json()
                self.rocon_logger.debug('response data > \n{}'.format(json_data), module_keyword=BT_KEYWORD)
                # await request.close()
                return json_data

        except Exception as exc:
            self.rocon_logger.error(exc)