import asyncio
import pydash
from rocon_client_sdk_py.core_logic.action import Action
from rocon_client_sdk_py.virtual_core.path_planner import PathPlanner


class Dock(Action):
    def __init__(self):
        super().__init__()
        self.name = 'Dock'
        self.func_name = 'dock'

    async def on_define(self, context):
        self.rocon_logger.debug('define the action of ' + self.name)
        api_config = context.api_configuration
        result = await api_config.get_stations()

        domain_station = []

        def cb(station):
            domain_station.append({'alias': station['name']+'('+str(station['marker_value'])+')', 'value': station['id']})

        pydash.map_(result, cb)

        return {
            'name': self.name,
            'func_name': self.func_name,
            'args': [
                {
                    'key': 'station',
                    'type': 'number',
                    'default': domain_station[len(domain_station) -1],
                    'domain': domain_station
                }
            ]
        }

    async def on_perform(self, context, args):
        station_id = pydash.find(args, {'key': 'station'})['value']
        station = await context.api_configuration.get_stations(station_id)

        if station is None:
            self.rocon_logger.debug('failed to get station')

        worker_content = context.blackboard.get_worker_content()
        worker_location = pydash.get(worker_content, 'type_specific.location')
        path_planner = PathPlanner(context)

        path = path_planner.get_path(worker_location['map'], worker_location['pose2d'], station['pose'])
        trajectory = path_planner.path_to_trajectory(worker_location, path)

        self.rocon_logger.debug('start to moving robot on path')

        for point in trajectory:
            worker_content = context.blackboard.get_worker_content()
            updated_type_specific = worker_content['type_specific']
            if 'theta' in point:
                pass
            else:
                point['theta'] = pydash.get(worker_content, 'type_specific.location.pose2d.theta')

            updated_type_specific['location'] = pydash.assign({}, updated_type_specific['location'], {
                'map': worker_location['map'],
                'pose2d': point
            })

            context.blackboard.set_worker_content({'type_specific': updated_type_specific})
            await context.blackboard.sync_worker()
            await asyncio.sleep(1)

        updated_type_specific['location']['pose2d']['theta'] = station['pose']['theta']
        context.blackboard.set_worker_content({'type_specific': updated_type_specific})
        await context.blackboard.sync_worker()
        await asyncio.sleep(1)
        return True
