import pydash
from rocon_client_sdk_py.core_logic.action import Action
from rocon_client_sdk_py.utils.waiter import Waiter
from rocon_client_sdk_py.virtual_core.components.actuators import Actuator

class Lock(Action):
    def __init__(self):
        super().__init__()
        self.name = 'Lock'
        self.func_name = 'lock'

    async def on_define(self, context):
        self.rocon_logger.debug('define the action of ' + self.name)
        api_config = context.api_configuration
        result = await api_config.get_resources()

        exclusive_area = pydash.get(result, 'ExclusiveArea')

        resource_list = []
        def cb(resource):
            resource_list.append(
                {'alias': resource['name'], 'value': resource['id']})

        pydash.map_(exclusive_area, cb)

        return {
            'name': self.name,
            'func_name': self.func_name,
            'args': [
                {
                    'type': 'string',
                    'key': 'resource',
                    'default': resource_list[0],
                    'domain': resource_list,
                    'options':{'user_input': False}
                }
            ]
        }

    async def on_perform(self, context, args):

        args_resource = pydash.find(args, {'key':'resource'})
        target_resource_id = pydash.get(args_resource, 'value')
        arg_properties = pydash.get(args_resource, 'properties')
        aligns = pydash.find(arg_properties, {'key': 'align'})

        enter_pose = pydash.pick(pydash.get(aligns, 'value.enter'), ['x', 'y'])

        MAX_RETRY = 100
        REQUEST_INTERVAL = 100
        work_id = context.blackboard.get_worker_content()['id']

        actuator = Actuator()
        await actuator.init_path_planner(context)
        await actuator.moving(context, enter_pose)

        waiter = Waiter(REQUEST_INTERVAL)
        for i in range(MAX_RETRY):
            self.rocon_logger.debug('check resource have assigned to this worker {}/{}'.format(i+1, MAX_RETRY))
            target_resource = await context.api_configuration.get_resource(target_resource_id)
            target_slots = target_resource['resource_slots']
            occupied = pydash.find(target_slots, {'user_id': work_id, 'status':'occupied'})
            if occupied:
                self.rocon_logger.debug('confirmed : resource have assigned to this worker')
                return True

            await waiter.wait()

        self.rocon_logger.debug('exceed maximum try to check resource have assigned to this worker')
        return False