import pydash
from rocon_client_sdk_py.core_logic.action import Action
from rocon_client_sdk_py.virtual_core.components.actuators import Actuator
from rocon_client_sdk_py.utils.waiter import Waiter


class Unlock(Action):
    def __init__(self):
        super().__init__()
        self.name = 'Unlock'
        self.func_name = 'unlock'

    async def on_define(self, context):
        self.rocon_logger.debug('define the action of ' + self.name)
        api_config = context.api_configuration
        result = await api_config.get_resources()

        exclusive_area = pydash.get(result, 'ExclusiveArea')
        resource_list = []
        def cb(resource):
            resource_list.append(
                {'alias': resource['name'], 'value': resource['id']})

        pydash.map_(exclusive_area, cb)

        return {
            'name': self.name,
            'func_name': self.func_name,
            'args': [
                {
                    'type': 'string',
                    'key': 'resource',
                    'default': resource_list[0],
                    'domain': resource_list,
                    'options':{'user_input': False}
                },
                {
                    'type': 'object',
                    'key': 'quit',
                    'default': {},
                    'domain': [],
                    'options': {'user_input': False}
                }
            ]
        }

    async def on_perform(self, context, args):

        args_resource = pydash.find(args, {'key': 'resource'})
        target_resource_id = pydash.get(args_resource, 'value')
        arg_properties = pydash.get(args_resource, 'properties')
        aligns = pydash.find(arg_properties, {'key': 'align'})

        exit_pose = pydash.pick(pydash.get(aligns, 'value.exit'), ['x', 'y'])

        MAX_RETRY = 100
        REQUEST_INTERVAL = 100
        worker_content = context.blackboard.get_worker_content()
        worker_id = worker_content['id']

        actuator = Actuator()
        await actuator.init_path_planner(context)
        await actuator.moving(context, exit_pose)

        waiter = Waiter(REQUEST_INTERVAL)
        for i in range(MAX_RETRY):
            self.rocon_logger.debug('return occupied resuorce {}/{}'.format(i+1, MAX_RETRY))
            target_resource = await context.api_configuration.get_resource(target_resource_id)
            target_slots = target_resource['resource_slots']
            occupied = pydash.find(target_slots, {'user_id': worker_id, 'status': 'occupied'})

            try:
                await context.api_configuration.return_resource(worker_id, target_resource_id, occupied['id'])
                return True
            except Exception as err:
                self.rocon_logger.error('failed to return resource with error', exception=err)

            await waiter.wait()

        self.rocon_logger.debug('exceed maximum try to return occupied resource')
        return False
