import asyncio
from rocon_client_sdk_py.logger.rocon_logger import rocon_logger


class AutodoorConsumer():
    def __init__(self):
        self.rocon_logger = rocon_logger
        pass

    async def request_open_autodoor(self, context, autodoor_id):
        RETRY = 10
        INTERVAL = 1000
        request = context.api_configuration.request

        worker = context.worker
        iot_op_url = worker._configs['temp']['iot_operator']
        url = 'http://{}/facilities/autodoors/{}/commands'.format(iot_op_url, autodoor_id)
        for i in range(RETRY):
            try:
                async with request.post(url, json={'action': 'open'}) as r:
                    if r.status == 200:
                        self.rocon_logger.debug('Open Autodoor: {} requested'.format(autodoor_id))
                        return True

            except Exception as exc:
                self.rocon_logger.error('there is problem while request open autodoor')
                self.rocon_logger.error('Exception occurred', exception=exc)

            await asyncio.sleep(INTERVAL/1000)

        return False

    async def request_close_autodoor(self, context, autodoor_id):
        RETRY = 10
        INTERVAL = 1000
        request = context.api_configuration.request

        worker = context.worker
        iot_op_url = worker._configs['temp']['iot_operator']
        url = 'http://{}/facilities/autodoors/{}/commands'.format(iot_op_url, autodoor_id)
        for i in range(RETRY):
            try:
                async with request.post(url, json={'action': 'close'}) as r:
                    if r.status == 200:
                        self.rocon_logger.debug('Close Autodoor: {} requested'.format(autodoor_id))
                        return True

            except Exception as exc:
                self.rocon_logger.error('there is problem while request close autodoor')
                self.rocon_logger.error('Exception occurred', exception=exc)


            await asyncio.sleep(INTERVAL/1000)

        return False

    async def ensure_autodoor_opened(self, context, autodoor_id):
        RETRY = 20
        INTERVAL = 1000
        request = context.api_configuration.request

        worker = context.worker
        iot_op_url = worker._configs['temp']['iot_operator']
        url = 'http://{}/facilities/autodoors/{}'.format(iot_op_url, autodoor_id)
        for i in range(RETRY):
            try:
                async with request.get(url) as r:
                    if r.status == 200:
                        json_data = await r.json()
                        if json_data['status'] == 'OPENED':
                            return True

                        self.rocon_logger.debug('Status of Autodoor: {} is {} it\'s not match with opended'.format(autodoor_id, json_data))

            except Exception as exc:
                self.rocon_logger.error('there is problem while checking autodoor is open or not')
                self.rocon_logger.error('Exception occurred', exception=exc)


            self.rocon_logger.debug('Check Autodoor : {} open 1000ms later'.format(autodoor_id))
            await asyncio.sleep(INTERVAL / 1000)

        self.rocon_logger.debug('failed to waiting autodoor, exceed timeout')
        return False

    async def ensure_autodoor_closed(self, context, autodoor_id):
        RETRY = 20
        INTERVAL = 1000
        request = context.api_configuration.request
        worker = context.worker
        iot_op_url = worker._configs['temp']['iot_operator']
        url = 'http://{}/facilities/autodoors/{}'.format(iot_op_url, autodoor_id)


        for i in range(RETRY):
            try:
                async with request.get(url) as r:
                    if r.status == 200:
                        json_data = await r.json()
                        if json_data['status'] == 'CLOSED':
                            return True

                        self.rocon_logger.debug('Status of Autodoor: {} is {} it\'s not match with opended'.format(autodoor_id, json_data))

            except Exception as exc:
                self.rocon_logger.error('there is problem while checking autodoor is open or not')
                self.rocon_logger.error('Exception occurred', exception=exc)


            self.rocon_logger.debug('Check Audtodoor: {} open 1000ms later'.format(autodoor_id))
            await asyncio.sleep(INTERVAL / 1000)

        self.rocon_logger.debug('failed to waiting autodoor, exceed timeout')
        return False
