import json
from .action_manager import ActionManager
from .hook_manager import HookManager
from .message_manager import MessageManager
from .trees.blackboard_manager import BlackboardManager
from .map_manager import MapManager
from rocon_client_sdk_py.rocon_apis import ApiTask
from rocon_client_sdk_py.rocon_apis import ApiWorker
from rocon_client_sdk_py.rocon_apis import ApiWorkerPool
from rocon_client_sdk_py.rocon_apis.api_report import ApiReport
from rocon_client_sdk_py.rocon_apis.api_configuration import ApiConfiguration
from rocon_client_sdk_py.rocon_apis.api_site_configuration import ApiSiteConfiguration


class Context():
    def __init__(self, worker, event_loop):
        worker.rocon_logger.debug('initializing Context')
        self._worker = worker
        self._is_booted = False

        self._hook_mgr = HookManager(self)
        self._act_mgr = ActionManager(self)
        self._msg_mgr = MessageManager(self)
        self._blackboard_manager = None
        self._event_loop = event_loop

        self._map_mgr = MapManager(self)

    @property
    def api_report(self) -> ApiReport:
        return self._worker.httpclient.api_report

    @property
    def api_task(self) -> ApiTask:
        return self._worker.httpclient.api_task

    @property
    def api_worker(self) -> ApiWorker:
        return self._worker.httpclient.api_worker

    @property
    def api_worker_pool(self) -> ApiWorkerPool:
        return self._worker.httpclient.api_worker_pool

    @property
    def api_configuration(self) -> ApiConfiguration:
        return self._worker.httpclient.api_config

    @property
    def api_site_configuration(self) -> ApiSiteConfiguration:
        return self._worker.httpclient.api_site_config

    @property
    def event_loop(self):
        return self._event_loop

    @property
    def worker(self):
        return self._worker

    @property
    def action_manager(self):
        return self._act_mgr

    @property
    def blackboard(self):
        return self._blackboard_manager

    @property
    def message_manager(self):
        return self._msg_mgr

    @property
    def hook_manager(self):
        return self._hook_mgr

    @property
    def map_manager(self):
        return self._map_mgr

    @blackboard.setter
    def blackboard(self, blackboard_manager:BlackboardManager):
        self._blackboard_manager = blackboard_manager

    @property
    def is_booted(self):
        return self._is_booted

    @is_booted.setter
    def is_booted(self, value:bool):
        self._is_booted = value

    @property
    def rocon_logger(self):
        return self.worker.rocon_logger

    def get_trees_stringify(self):

        trees_data ={
            'treeId': str(self.worker.tree_manager.get_tree_id()),
            'tree': self.worker.tree_manager.get_trees_info(),
            'bb': self.blackboard.get_blackborad_info()
        }

        json_data = json.dumps(trees_data)

        #json_data = '{"treeId":"1ac7f790-0cb2-4855-b72a-3dba48c6b5e1","tree":{"id":"e34f02c2-653b-45e7-9a13-bf83df1c29d9","category":"composite","type":"Sequence","name":"rootTree","description":"","properties":{},"children":[{"id":"67fde38e-a12b-4af0-836b-304e746aa5a1","category":"decorator","type":"UntilSuccess","name":"Until Success","description":"","properties":{"maxLoop":-1},"child":{"id":"9cf3262b-8bbe-4abb-9720-b32579334ed2","category":"composite","type":"Priority","name":"bootstrap","description":"","properties":{},"children":[{"id":"5656c1b7-7564-4237-b10e-4d63e2a630c3","category":"task","type":"Task","name":"isBooted","description":"","properties":{}},{"id":"d26af1d1-8db2-4eab-9985-3537d3dc6a07","category":"composite","type":"Sequence","name":"bootstraping","description":"","properties":{},"children":[{"id":"7fbb96fd-0a9d-48fb-8e94-23cd9366dfc0","category":"task","type":"AsyncTask","name":"initWorker","description":"","properties":{}},{"id":"d7bdf450-fcaa-4c92-ba43-968adf673e30","category":"task","type":"AsyncTask","name":"initTask","description":"","properties":{}},{"id":"d53dfa94-cbc7-470f-93c6-aa70b657e52a","category":"task","type":"AsyncTask","name":"bootCheck","description":"","properties":{}}]}]},"maxLoop":-1},{"id":"d92be6bf-04f9-421f-bae6-2a6796e19cea","category":"decorator","type":"Repeater","name":"Repeat <maxLoop>x","description":"","properties":{"maxLoop":-1},"child":{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},"maxLoop":-1}]},"bb":{"trees":{"1ac7f790-0cb2-4855-b72a-3dba48c6b5e1":{"tree":{},"treeMemory":{},"node":{"e34f02c2-653b-45e7-9a13-bf83df1c29d9":{"isOpen":true,"runningChild":1,"lastStatus":"RUNNING"},"67fde38e-a12b-4af0-836b-304e746aa5a1":{"isOpen":false,"i":0,"lastStatus":"SUCCESS"},"9cf3262b-8bbe-4abb-9720-b32579334ed2":{"isOpen":false,"runningChild":0,"lastStatus":"SUCCESS"},"5656c1b7-7564-4237-b10e-4d63e2a630c3":{"isOpen":false,"lastStatus":"SUCCESS"},"d26af1d1-8db2-4eab-9985-3537d3dc6a07":{"isOpen":false,"runningChild":2,"lastStatus":"SUCCESS"},"7fbb96fd-0a9d-48fb-8e94-23cd9366dfc0":{"isOpen":false,"lastStatus":"SUCCESS"},"d7bdf450-fcaa-4c92-ba43-968adf673e30":{"isOpen":false,"lastStatus":"SUCCESS"},"d53dfa94-cbc7-470f-93c6-aa70b657e52a":{"isOpen":false,"lastStatus":"SUCCESS"},"d92be6bf-04f9-421f-bae6-2a6796e19cea":{"isOpen":true,"i":0,"lastStatus":"RUNNING"},"6d4df051-e6b4-4799-9eaf-5af8f9be45ef":{"isOpen":true,"runningChild":1,"lastStatus":"RUNNING"},"68db8889-b3fc-4c64-a3c3-43952e66889d":{"isOpen":false,"runningChild":0,"lastStatus":"SUCCESS"},"2687bf8e-078a-45b7-9819-12683d940516":{"isOpen":false,"lastStatus":"SUCCESS"},"ad061e25-5528-4435-9304-363e3e14a475":{"isOpen":false,"lastStatus":"SUCCESS"},"2aaf2074-720f-4986-9cfb-337b812ba3f2":{"isOpen":true,"runningChild":0,"lastStatus":"RUNNING"},"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7":{"isOpen":true,"lastStatus":"RUNNING","asyncStatus":"RUNNING"},"68569def-7ddf-4c95-8ae0-f67ff4b13015":{"isOpen":false,"runningChild":0,"lastStatus":"FAILURE"},"6a48bef5-fd84-4312-8493-55c0898f9723":{"isOpen":false,"runningChild":2,"lastStatus":"FAILURE"},"0e43c429-f2a9-4056-8ae3-af9a7cac48cf":{"isOpen":false,"lastStatus":"SUCCESS"},"ca6851c6-3724-4df1-ae62-a42b0df14490":{"isOpen":false,"lastStatus":"SUCCESS"},"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5":{"isOpen":false,"runningChild":1,"lastStatus":"FAILURE"},"e34d6571-662b-4595-98ee-3422605976ec":{"isOpen":false,"lastStatus":"FAILURE"},"2568af36-d161-42d5-85b7-df69795da934":{"isOpen":false,"runningChild":0,"lastStatus":"FAILURE"},"c6029fe7-0c27-4a54-b2b9-d7c3a2571002":{"isOpen":false,"lastStatus":"FAILURE"}}}},"shared":{}}}'
        #json_data = '{"treeId":"1ac7f790-0cb2-4855-b72a-3dba48c6b5e1","tree":{"id":"e34f02c2-653b-45e7-9a13-bf83df1c29d9","category":"composite","type":"Sequence","name":"rootTree","description":"","properties":{},"children":[{"id":"67fde38e-a12b-4af0-836b-304e746aa5a1","category":"decorator","type":"UntilSuccess","name":"Until Success","description":"","properties":{"maxLoop":-1},"child":{"id":"9cf3262b-8bbe-4abb-9720-b32579334ed2","category":"composite","type":"Priority","name":"bootstrap","description":"","properties":{},"children":[{"id":"5656c1b7-7564-4237-b10e-4d63e2a630c3","category":"task","type":"Task","name":"isBooted","description":"","properties":{}},{"id":"d26af1d1-8db2-4eab-9985-3537d3dc6a07","category":"composite","type":"Sequence","name":"bootstraping","description":"","properties":{},"children":[{"id":"7fbb96fd-0a9d-48fb-8e94-23cd9366dfc0","category":"task","type":"AsyncTask","name":"initWorker","description":"","properties":{}},{"id":"d7bdf450-fcaa-4c92-ba43-968adf673e30","category":"task","type":"AsyncTask","name":"initTask","description":"","properties":{}},{"id":"d53dfa94-cbc7-470f-93c6-aa70b657e52a","category":"task","type":"AsyncTask","name":"bootCheck","description":"","properties":{}}]}]},"maxLoop":-1},{"id":"d92be6bf-04f9-421f-bae6-2a6796e19cea","category":"decorator","type":"Repeater","name":"Repeat <maxLoop>x","description":"","properties":{"maxLoop":-1},"child":{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},"maxLoop":-1}]},"bb":{"trees":{"1ac7f790-0cb2-4855-b72a-3dba48c6b5e1":{"tree":{},"treeMemory":{"configs":{"hostname":{"concert":"localhost:10602","siteConfig":"localhost:10604"},"server":{"port":10999},"logger":{"console_level":"info","console_json":true,"file_level":"info"},"assets":{"base":"./assets","map":"/maps"},"vw_archives":"./vw_archives","temp":{"iot_operator":"localhost:10702"},"action":{"move":"direct"},"taskName":"Virtual Delivery","taskDescription":"mocking GoCart Delivery in virtual environment"},"uuid":"a1","actionDefinePaths":"/home/blcho/dev/virtual_core/src/core/VirtualWorker/actions","messageDefinePaths":"/home/blcho/dev/virtual_core/src/core/VirtualWorker/messageHandlers","tickInterval":1000,"updateInterval":2000,"activeNodes":[{"id":"e34f02c2-653b-45e7-9a13-bf83df1c29d9","category":"composite","type":"Sequence","name":"rootTree","description":"","properties":{},"children":[{"id":"67fde38e-a12b-4af0-836b-304e746aa5a1","category":"decorator","type":"UntilSuccess","name":"Until Success","description":"","properties":{"maxLoop":-1},"child":{"id":"9cf3262b-8bbe-4abb-9720-b32579334ed2","category":"composite","type":"Priority","name":"bootstrap","description":"","properties":{},"children":[{"id":"5656c1b7-7564-4237-b10e-4d63e2a630c3","category":"task","type":"Task","name":"isBooted","description":"","properties":{}},{"id":"d26af1d1-8db2-4eab-9985-3537d3dc6a07","category":"composite","type":"Sequence","name":"bootstraping","description":"","properties":{},"children":[{"id":"7fbb96fd-0a9d-48fb-8e94-23cd9366dfc0","category":"task","type":"AsyncTask","name":"initWorker","description":"","properties":{}},{"id":"d7bdf450-fcaa-4c92-ba43-968adf673e30","category":"task","type":"AsyncTask","name":"initTask","description":"","properties":{}},{"id":"d53dfa94-cbc7-470f-93c6-aa70b657e52a","category":"task","type":"AsyncTask","name":"bootCheck","description":"","properties":{}}]}]},"maxLoop":-1},{"id":"d92be6bf-04f9-421f-bae6-2a6796e19cea","category":"decorator","type":"Repeater","name":"Repeat <maxLoop>x","description":"","properties":{"maxLoop":-1},"child":{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},"maxLoop":-1}]},{"id":"d92be6bf-04f9-421f-bae6-2a6796e19cea","category":"decorator","type":"Repeater","name":"Repeat <maxLoop>x","description":"","properties":{"maxLoop":-1},"child":{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},"maxLoop":-1},{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}],"worker":{"capability":{"tasks":[]},"type":"unknown worker type","release_version":"0.0.0","type_specific":{"battery":{"battery_level":75,"charging_status":0},"location":{"map":"5cd8ef9d65b9b770ac03bd1d","pose2d":{"x":13.68,"y":11.89,"theta":1.05},"semantic_location":null}},"_id":"5e5338bd1f90480013ec26f1","uuid":"a1","__v":0,"created_at":"2020-02-24T02:45:17.790Z","name":"VirtualWorker(a1)","updated_at":"2020-02-25T00:43:53.129Z","status":"offline","id":"5e5338bd1f90480013ec26f1"},"task":{"_id":"5e4e11f696d5900e0f298a96","actions_hash":"33143578c290ef33b280aa5a61048f5d5915c35b4c809edf709c5485ea314513","name":"Virtual Delivery","__v":0,"actions":[{"fixed":false,"args":[{"type":"number","format":null,"value":null,"domain":[{"alias":"VW Docking Station 2(32)","value":"5cd8f0e5ab582e775cd6ca66"},{"alias":"VW Docking Station 3(33)","value":"5cd8f0f7ab582e775cd6ca67"},{"alias":"Cafe Robot Docking Station(11)","value":"5cd8f1b0ab582e775cd6ca6c"},{"alias":"VW Docking Station 1(31)","value":"5dc253a295a03d00127c794b"}],"default":{"alias":"VW Docking Station 1(31)","value":"5dc253a295a03d00127c794b"},"properties":[],"key":"station"}],"func_name":"dock"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},{"alias":"intersection test area","value":"5dbfa93395a03d00127c78d3"},{"alias":"maze to table 2","value":"5dc3a8e795a03d00127c7959"}],"default":{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},"properties":[],"key":"resource","options":{"user_input":false}}],"func_name":"lock"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},{"alias":"In Front Of Cleaning Robot Team","value":"5cd8f059ab582e775cd6ca62"},{"alias":"In Front Of Military Robot Team","value":"5cd8f0a8ab582e775cd6ca63"},{"alias":"In Front Of QA Team","value":"5cd8f0b4ab582e775cd6ca64"},{"alias":"In Front Of Information desk","value":"5cd8f0ccab582e775cd6ca65"},{"alias":"R&D Center Entrance","value":"5cd8f10eab582e775cd6ca68"},{"alias":"Table 1","value":"5cd8f1b9ab582e775cd6ca6d"},{"alias":"Table 2","value":"5cd8f1c2ab582e775cd6ca6e"},{"alias":"Table 3","value":"5cd8f1c8ab582e775cd6ca6f"},{"alias":"Head Office Entrance","value":"5cd8f21aab582e775cd6ca72"},{"alias":"In Front of Meeting Room","value":"5cd8f22fab582e775cd6ca73"},{"alias":"W_E4_1","value":"5cf4f9c8f057bd4671f4d6a5"},{"alias":"W_E4_2","value":"5cf4f9e2f057bd4671f4d6a6"},{"alias":"W_E3_1","value":"5cf4fa25f057bd4671f4d6a7"},{"alias":"W_E3_2","value":"5cf4fa30f057bd4671f4d6a8"},{"alias":"W_E1_1","value":"5cf4fa4ef057bd4671f4d6a9"},{"alias":"W_E1_2","value":"5cf4fa5ef057bd4671f4d6aa"},{"alias":"W_X3_1","value":"5d01a8d3a25bd45942161731"},{"alias":"W_X3_2","value":"5d01a8fda25bd45942161732"},{"alias":"W_X3_3","value":"5d01a923a25bd45942161733"},{"alias":"W_X3_4","value":"5d01a930a25bd45942161734"},{"alias":"3F hall","value":"5d09969e13197f78beb339e8"},{"alias":"W_A3_1","value":"5d099d6413197f78beb339e9"},{"alias":"W_A3_2","value":"5d099d8313197f78beb339ea"},{"alias":"W_A3_3","value":"5d099da113197f78beb339eb"},{"alias":"W_A3_4","value":"5d099dac13197f78beb339ec"},{"alias":"Cafe Home","value":"5d3154df7a86d200120faf05"},{"alias":"Cafe Docking Front","value":"5d3154f97a86d200120faf06"},{"alias":"inside intersection","value":"5dbfa95295a03d00127c78d4"},{"alias":"intersection A waiting 1","value":"5dbfa97795a03d00127c78d5"},{"alias":"intersection B waiting 1","value":"5dbfa98d95a03d00127c78d6"},{"alias":"intersection C waiting 1","value":"5dbfb07895a03d00127c78d8"},{"alias":"intersection B side","value":"5dbfb10295a03d00127c78da"},{"alias":"intersection C side","value":"5dbfb11095a03d00127c78db"},{"alias":"intersection A side","value":"5dbfb54195a03d00127c78dc"},{"alias":"W_X3_5","value":"5dc0cbe395a03d00127c78df"},{"alias":"W_X3_6","value":"5dc0cbf295a03d00127c78e0"},{"alias":"intersection A waiting 2","value":"5dc2208895a03d00127c7940"},{"alias":"intersection A waiting 3","value":"5dc220f195a03d00127c7941"},{"alias":"intersection B waiting 2","value":"5dc2212f95a03d00127c7942"},{"alias":"intersection B waiting 3","value":"5dc2215b95a03d00127c7943"},{"alias":"intersection C waiting 2","value":"5dc221a895a03d00127c7944"},{"alias":"intersection C waiting 3","value":"5dc221b995a03d00127c7945"},{"alias":"intersection D side","value":"5dc221cc95a03d00127c7946"},{"alias":"intersection D waiting 1","value":"5dc221e095a03d00127c7947"},{"alias":"intersection D waiting 2","value":"5dc221f095a03d00127c7948"},{"alias":"intersection D waiting 3","value":"5dc2220b95a03d00127c7949"},{"alias":"3F Docking Front 1","value":"5dc2580f95a03d00127c794c"},{"alias":"3F Docking Front2","value":"5dc2582195a03d00127c794d"},{"alias":"3F Docking Front 3","value":"5dc2583795a03d00127c794e"},{"alias":"W_A3_5","value":"5dc3a69e95a03d00127c794f"},{"alias":"W_A3_6","value":"5dc3a6f195a03d00127c7950"},{"alias":"W_E3_3","value":"5dc3a72495a03d00127c7951"},{"alias":"W_E1_3","value":"5dc3a75095a03d00127c7952"},{"alias":"MAZE_W1","value":"5dc3a91295a03d00127c795a"},{"alias":"MAZE_W2","value":"5dc3a92295a03d00127c795b"},{"alias":"MAZE_W3","value":"5dc3a93195a03d00127c795c"}],"default":{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},"properties":[],"key":"destination","options":{"min":0,"user_input":false,"max":0}},{"type":"boolean","format":null,"value":null,"domain":[{"alias":"True","value":true},{"alias":"False","value":false}],"default":{"alias":"True","value":true},"properties":[],"key":"goal_finishing"},{"type":"number","format":null,"value":null,"domain":[0,10,20,30,40,50,60],"default":30,"properties":[],"key":"finishing_timeout","options":{"user_input":true,"min":0,"max":300}}],"func_name":"move"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[],"default":null,"properties":[],"key":"default_sender","name":"Default Sender","options":{"user_input":true}},{"type":"string","format":null,"value":null,"domain":[],"default":null,"properties":[],"key":"default_message","name":"Default Message","options":{"user_input":true}},{"type":"object","format":null,"value":null,"domain":[],"default":null,"properties":[{"key":"email","name":"E-mail","type":"boolean","default":true},{"key":"sms","name":"SMS","type":"boolean","default":false},{"key":"web_push","name":"Web_push","type":"boolean","default":false}],"key":"default_method","name":"Default Method"},{"type":"object","format":null,"value":null,"domain":[],"default":null,"properties":[{"key":"custom_sender","name":"Custom Sender","domain":[],"type":"string","default":null,"options":{"user_input":true}},{"key":"custom_message","name":"Custom Message","domain":[],"type":"string","default":null,"options":{"user_input":true}},{"key":"custom_method","name":"Custom Method","type":"object","properties":[{"key":"email","name":"E-mail","type":"boolean","default":true},{"key":"sms","name":"SMS","type":"boolean","default":false},{"key":"web_push","name":"Web_push","type":"boolean","default":false}]},{"key":"receiver_id","name":"Receiver ID","domain":[],"type":"string","default":null,"options":{"user_input":true}},{"key":"receiver_type","name":"Receiver Type","domain":[],"type":"string","default":null,"options":{"user_input":true}}],"key":"receivers","name":"Receiver"}],"func_name":"notify"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},{"alias":"intersection test area","value":"5dbfa93395a03d00127c78d3"},{"alias":"maze to table 2","value":"5dc3a8e795a03d00127c7959"}],"default":{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},"properties":[],"key":"resource","options":{"user_input":false}},{"type":"object","format":null,"value":null,"domain":[],"default":null,"properties":[],"key":"quit","options":{"user_input":false}}],"func_name":"unlock"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},{"alias":"In Front Of Cleaning Robot Team","value":"5cd8f059ab582e775cd6ca62"},{"alias":"In Front Of Military Robot Team","value":"5cd8f0a8ab582e775cd6ca63"},{"alias":"In Front Of QA Team","value":"5cd8f0b4ab582e775cd6ca64"},{"alias":"In Front Of Information desk","value":"5cd8f0ccab582e775cd6ca65"},{"alias":"R&D Center Entrance","value":"5cd8f10eab582e775cd6ca68"},{"alias":"Table 1","value":"5cd8f1b9ab582e775cd6ca6d"},{"alias":"Table 2","value":"5cd8f1c2ab582e775cd6ca6e"},{"alias":"Table 3","value":"5cd8f1c8ab582e775cd6ca6f"},{"alias":"Head Office Entrance","value":"5cd8f21aab582e775cd6ca72"},{"alias":"In Front of Meeting Room","value":"5cd8f22fab582e775cd6ca73"},{"alias":"W_E4_1","value":"5cf4f9c8f057bd4671f4d6a5"},{"alias":"W_E4_2","value":"5cf4f9e2f057bd4671f4d6a6"},{"alias":"W_E3_1","value":"5cf4fa25f057bd4671f4d6a7"},{"alias":"W_E3_2","value":"5cf4fa30f057bd4671f4d6a8"},{"alias":"W_E1_1","value":"5cf4fa4ef057bd4671f4d6a9"},{"alias":"W_E1_2","value":"5cf4fa5ef057bd4671f4d6aa"},{"alias":"W_X3_1","value":"5d01a8d3a25bd45942161731"},{"alias":"W_X3_2","value":"5d01a8fda25bd45942161732"},{"alias":"W_X3_3","value":"5d01a923a25bd45942161733"},{"alias":"W_X3_4","value":"5d01a930a25bd45942161734"},{"alias":"3F hall","value":"5d09969e13197f78beb339e8"},{"alias":"W_A3_1","value":"5d099d6413197f78beb339e9"},{"alias":"W_A3_2","value":"5d099d8313197f78beb339ea"},{"alias":"W_A3_3","value":"5d099da113197f78beb339eb"},{"alias":"W_A3_4","value":"5d099dac13197f78beb339ec"},{"alias":"Cafe Home","value":"5d3154df7a86d200120faf05"},{"alias":"Cafe Docking Front","value":"5d3154f97a86d200120faf06"},{"alias":"inside intersection","value":"5dbfa95295a03d00127c78d4"},{"alias":"intersection A waiting 1","value":"5dbfa97795a03d00127c78d5"},{"alias":"intersection B waiting 1","value":"5dbfa98d95a03d00127c78d6"},{"alias":"intersection C waiting 1","value":"5dbfb07895a03d00127c78d8"},{"alias":"intersection B side","value":"5dbfb10295a03d00127c78da"},{"alias":"intersection C side","value":"5dbfb11095a03d00127c78db"},{"alias":"intersection A side","value":"5dbfb54195a03d00127c78dc"},{"alias":"W_X3_5","value":"5dc0cbe395a03d00127c78df"},{"alias":"W_X3_6","value":"5dc0cbf295a03d00127c78e0"},{"alias":"intersection A waiting 2","value":"5dc2208895a03d00127c7940"},{"alias":"intersection A waiting 3","value":"5dc220f195a03d00127c7941"},{"alias":"intersection B waiting 2","value":"5dc2212f95a03d00127c7942"},{"alias":"intersection B waiting 3","value":"5dc2215b95a03d00127c7943"},{"alias":"intersection C waiting 2","value":"5dc221a895a03d00127c7944"},{"alias":"intersection C waiting 3","value":"5dc221b995a03d00127c7945"},{"alias":"intersection D side","value":"5dc221cc95a03d00127c7946"},{"alias":"intersection D waiting 1","value":"5dc221e095a03d00127c7947"},{"alias":"intersection D waiting 2","value":"5dc221f095a03d00127c7948"},{"alias":"intersection D waiting 3","value":"5dc2220b95a03d00127c7949"},{"alias":"3F Docking Front 1","value":"5dc2580f95a03d00127c794c"},{"alias":"3F Docking Front2","value":"5dc2582195a03d00127c794d"},{"alias":"3F Docking Front 3","value":"5dc2583795a03d00127c794e"},{"alias":"W_A3_5","value":"5dc3a69e95a03d00127c794f"},{"alias":"W_A3_6","value":"5dc3a6f195a03d00127c7950"},{"alias":"W_E3_3","value":"5dc3a72495a03d00127c7951"},{"alias":"W_E1_3","value":"5dc3a75095a03d00127c7952"},{"alias":"MAZE_W1","value":"5dc3a91295a03d00127c795a"},{"alias":"MAZE_W2","value":"5dc3a92295a03d00127c795b"},{"alias":"MAZE_W3","value":"5dc3a93195a03d00127c795c"},{"alias":"1f_prohibited","value":"5cf4f688f057bd4671f4d69d"},{"alias":"1f_prohibited","value":"5cf4f6aff057bd4671f4d69e"},{"alias":"1f_prohibited","value":"5cf4f6dbf057bd4671f4d69f"},{"alias":"1f_prohibited","value":"5cf4f6f9f057bd4671f4d6a0"},{"alias":"1f_prohibited","value":"5cf4f70bf057bd4671f4d6a1"},{"alias":"1f_prohibited","value":"5cf4f736f057bd4671f4d6a2"},{"alias":"3f_prohibited","value":"5cf4f7c5f057bd4671f4d6a3"},{"alias":"3f_prohibited","value":"5cf4f7f7f057bd4671f4d6a4"},{"alias":"1f_prohibited","value":"5cf4fa95f057bd4671f4d6ab"},{"alias":"4f_zone_prohibit_backdoor","value":"5dc3a78895a03d00127c7953"},{"alias":"1f_zone_prohibie_maze1","value":"5dc3a7e395a03d00127c7954"},{"alias":"1f_zone_prohibie_maze2","value":"5dc3a80595a03d00127c7955"},{"alias":"1f_zone_prohibie_maze3","value":"5dc3a83295a03d00127c7956"},{"alias":"1f_zone_prohibie_maze4","value":"5dc3a85595a03d00127c7957"},{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},{"alias":"intersection test area","value":"5dbfa93395a03d00127c78d3"},{"alias":"maze to table 2","value":"5dc3a8e795a03d00127c7959"},{"alias":"R&D Autodoor","value":"5cd8f281ab582e775cd6ca81"},{"alias":"VW Docking Station 2","value":"5cd8f0e5ab582e775cd6ca66"},{"alias":"VW Docking Station 3","value":"5cd8f0f7ab582e775cd6ca67"},{"alias":"Cafe Robot Docking Station","value":"5cd8f1b0ab582e775cd6ca6c"},{"alias":"VW Docking Station 1","value":"5dc253a295a03d00127c794b"},{"alias":"Center Elevator","value":"5cd8f246ab582e775cd6ca74"},{"alias":"Center Elevator 1F","value":"5cd8f256ab582e775cd6ca75"},{"alias":"Center Elevator 3F","value":"5cd8f266ab582e775cd6ca76"},{"alias":"Center Elevator 4F","value":"5cd8f271ab582e775cd6ca77"},{"alias":"test_pro","value":"5e45fe57479c5a0019825585"}],"default":{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},"properties":[],"key":"resource_id"},{"type":"string","format":null,"value":null,"domain":[{"alias":"ExclusiveArea","value":"ExclusiveArea"},{"alias":"Autodoor","value":"Autodoor"},{"alias":"Elevator","value":"Teleporter"}],"default":{"alias":"ExclusiveArea","value":"ExclusiveArea"},"properties":[],"key":"resource_type"},{"type":"object","format":null,"value":null,"domain":[{}],"default":null,"properties":[],"key":"params"}],"func_name":"use_resource"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"Duration","value":"duration"},{"alias":"Controlled by system","value":"signal"},{"alias":"Until button press","value":"human_input"}],"default":"human_input","properties":[],"key":"type","options":{"user_input":false,"regex":null}},{"type":"number","format":null,"value":null,"domain":[{"alias":"forever","value":"-1"},{"alias":"10000 ms","value":"10000"},{"alias":"20000 ms","value":"20000"},{"alias":"30000 ms","value":"30000"},{"alias":"40000 ms","value":"40000"},{"alias":"50000 ms","value":"50000"},{"alias":"60000 ms","value":"60000"}],"default":30000,"properties":[],"key":"timeout","options":{"user_input":true,"regex":null}},{"type":"string","format":null,"value":null,"domain":["revision","success","fail"],"default":"revision","properties":[],"key":"param"},{"type":"boolean","format":null,"value":null,"domain":[{"alias":"True","value":true},{"alias":"False","value":false}],"default":{"alias":"False","value":false},"properties":[],"key":"door_behavior"}],"func_name":"wait"}],"created_at":"2020-02-20T04:58:30.227Z","description":"mocking GoCart Delivery in virtual environment","updated_at":"2020-02-25T00:43:45.123Z","id":"5e4e11f696d5900e0f298a96"},"bootstrap":true,"worker_update":{"updated_at":"2020-02-25T00:43:57.190Z"},"status":"idle","task_updated":1582591425096},"node":{"e34f02c2-653b-45e7-9a13-bf83df1c29d9":{"isOpen":true,"runningChild":1,"lastStatus":"RUNNING"},"67fde38e-a12b-4af0-836b-304e746aa5a1":{"isOpen":false,"i":0,"lastStatus":"SUCCESS"},"9cf3262b-8bbe-4abb-9720-b32579334ed2":{"isOpen":false,"runningChild":0,"lastStatus":"SUCCESS"},"5656c1b7-7564-4237-b10e-4d63e2a630c3":{"isOpen":false,"lastStatus":"SUCCESS"},"d26af1d1-8db2-4eab-9985-3537d3dc6a07":{"isOpen":false,"runningChild":2,"lastStatus":"SUCCESS"},"7fbb96fd-0a9d-48fb-8e94-23cd9366dfc0":{"isOpen":false,"lastStatus":"SUCCESS"},"d7bdf450-fcaa-4c92-ba43-968adf673e30":{"isOpen":false,"lastStatus":"SUCCESS"},"d53dfa94-cbc7-470f-93c6-aa70b657e52a":{"isOpen":false,"lastStatus":"SUCCESS"},"d92be6bf-04f9-421f-bae6-2a6796e19cea":{"isOpen":true,"i":0,"lastStatus":"RUNNING"},"6d4df051-e6b4-4799-9eaf-5af8f9be45ef":{"isOpen":true,"runningChild":1,"lastStatus":"RUNNING"},"68db8889-b3fc-4c64-a3c3-43952e66889d":{"isOpen":false,"runningChild":0,"lastStatus":"SUCCESS"},"2687bf8e-078a-45b7-9819-12683d940516":{"isOpen":false,"lastStatus":"SUCCESS"},"ad061e25-5528-4435-9304-363e3e14a475":{"isOpen":false,"lastStatus":"SUCCESS"},"2aaf2074-720f-4986-9cfb-337b812ba3f2":{"isOpen":true,"runningChild":0,"lastStatus":"RUNNING"},"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7":{"isOpen":true,"lastStatus":"RUNNING","asyncStatus":"RUNNING"},"68569def-7ddf-4c95-8ae0-f67ff4b13015":{"isOpen":false,"runningChild":0,"lastStatus":"FAILURE"},"6a48bef5-fd84-4312-8493-55c0898f9723":{"isOpen":false,"runningChild":2,"lastStatus":"FAILURE"},"0e43c429-f2a9-4056-8ae3-af9a7cac48cf":{"isOpen":false,"lastStatus":"SUCCESS"},"ca6851c6-3724-4df1-ae62-a42b0df14490":{"isOpen":false,"lastStatus":"SUCCESS"},"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5":{"isOpen":false,"runningChild":1,"lastStatus":"FAILURE"},"e34d6571-662b-4595-98ee-3422605976ec":{"isOpen":false,"lastStatus":"FAILURE"},"2568af36-d161-42d5-85b7-df69795da934":{"isOpen":false,"runningChild":0,"lastStatus":"FAILURE"},"c6029fe7-0c27-4a54-b2b9-d7c3a2571002":{"isOpen":false,"lastStatus":"FAILURE"}}}},"shared":{}}}'
        #json_data = '{"treeId":"1ac7f790-0cb2-4855-b72a-3dba48c6b5e1","tree":{"id":"e34f02c2-653b-45e7-9a13-bf83df1c29d9","category":"compositexxx","type":"Sequence","name":"rootTree","description":"","properties":{},"children":[{"id":"67fde38e-a12b-4af0-836b-304e746aa5a1","category":"decorator","type":"UntilSuccess","name":"Until Success","description":"","properties":{"maxLoop":-1},"child":{"id":"9cf3262b-8bbe-4abb-9720-b32579334ed2","category":"composite","type":"Priority","name":"bootstrap","description":"","properties":{},"children":[{"id":"5656c1b7-7564-4237-b10e-4d63e2a630c3","category":"task","type":"Task","name":"isBooted","description":"","properties":{}},{"id":"d26af1d1-8db2-4eab-9985-3537d3dc6a07","category":"composite","type":"Sequence","name":"bootstraping","description":"","properties":{},"children":[{"id":"7fbb96fd-0a9d-48fb-8e94-23cd9366dfc0","category":"task","type":"AsyncTask","name":"initWorker","description":"","properties":{}},{"id":"d7bdf450-fcaa-4c92-ba43-968adf673e30","category":"task","type":"AsyncTask","name":"initTask","description":"","properties":{}},{"id":"d53dfa94-cbc7-470f-93c6-aa70b657e52a","category":"task","type":"AsyncTask","name":"bootCheck","description":"","properties":{}}]}]},"maxLoop":-1},{"id":"d92be6bf-04f9-421f-bae6-2a6796e19cea","category":"decorator","type":"Repeater","name":"Repeat <maxLoop>x","description":"","properties":{"maxLoop":-1},"child":{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},"maxLoop":-1}]},"bb":{"trees":{"1ac7f790-0cb2-4855-b72a-3dba48c6b5e1":{"tree":{"configs":{"hostname":{"concert":"localhost:10602","siteConfig":"localhost:10604"},"server":{"port":10999},"logger":{"console_level":"info","console_json":true,"file_level":"info"},"assets":{"base":"./assets","map":"/maps"},"vw_archives":"./vw_archives","temp":{"iot_operator":"localhost:10702"},"action":{"move":"direct"},"taskName":"Virtual Delivery","taskDescription":"mocking GoCart Delivery in virtual environment"},"uuid":"a1","actionDefinePaths":"/home/blcho/dev/virtual_core/src/core/VirtualWorker/actions","messageDefinePaths":"/home/blcho/dev/virtual_core/src/core/VirtualWorker/messageHandlers","tickInterval":1000,"updateInterval":2000,"activeNodes":[{"id":"e34f02c2-653b-45e7-9a13-bf83df1c29d9","category":"composite","type":"Sequence","name":"rootTree","description":"","properties":{},"children":[{"id":"67fde38e-a12b-4af0-836b-304e746aa5a1","category":"decorator","type":"UntilSuccess","name":"Until Success","description":"","properties":{"maxLoop":-1},"child":{"id":"9cf3262b-8bbe-4abb-9720-b32579334ed2","category":"composite","type":"Priority","name":"bootstrap","description":"","properties":{},"children":[{"id":"5656c1b7-7564-4237-b10e-4d63e2a630c3","category":"task","type":"Task","name":"isBooted","description":"","properties":{}},{"id":"d26af1d1-8db2-4eab-9985-3537d3dc6a07","category":"composite","type":"Sequence","name":"bootstraping","description":"","properties":{},"children":[{"id":"7fbb96fd-0a9d-48fb-8e94-23cd9366dfc0","category":"task","type":"AsyncTask","name":"initWorker","description":"","properties":{}},{"id":"d7bdf450-fcaa-4c92-ba43-968adf673e30","category":"task","type":"AsyncTask","name":"initTask","description":"","properties":{}},{"id":"d53dfa94-cbc7-470f-93c6-aa70b657e52a","category":"task","type":"AsyncTask","name":"bootCheck","description":"","properties":{}}]}]},"maxLoop":-1},{"id":"d92be6bf-04f9-421f-bae6-2a6796e19cea","category":"decorator","type":"Repeater","name":"Repeat <maxLoop>x","description":"","properties":{"maxLoop":-1},"child":{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},"maxLoop":-1}]},{"id":"d92be6bf-04f9-421f-bae6-2a6796e19cea","category":"decorator","type":"Repeater","name":"Repeat <maxLoop>x","description":"","properties":{"maxLoop":-1},"child":{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},"maxLoop":-1},{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}],"worker":{"capability":{"tasks":[]},"type":"unknown worker type","release_version":"0.0.0","type_specific":{"battery":{"battery_level":75,"charging_status":0},"location":{"map":"5cd8ef9d65b9b770ac03bd1d","pose2d":{"x":13.68,"y":11.89,"theta":1.05},"semantic_location":null}},"_id":"5e5338bd1f90480013ec26f1","uuid":"a1","__v":0,"created_at":"2020-02-24T02:45:17.790Z","name":"VirtualWorker(a1)","updated_at":"2020-02-25T00:43:53.129Z","status":"offline","id":"5e5338bd1f90480013ec26f1"},"task":{"_id":"5e4e11f696d5900e0f298a96","actions_hash":"33143578c290ef33b280aa5a61048f5d5915c35b4c809edf709c5485ea314513","name":"Virtual Delivery","__v":0,"actions":[{"fixed":false,"args":[{"type":"number","format":null,"value":null,"domain":[{"alias":"VW Docking Station 2(32)","value":"5cd8f0e5ab582e775cd6ca66"},{"alias":"VW Docking Station 3(33)","value":"5cd8f0f7ab582e775cd6ca67"},{"alias":"Cafe Robot Docking Station(11)","value":"5cd8f1b0ab582e775cd6ca6c"},{"alias":"VW Docking Station 1(31)","value":"5dc253a295a03d00127c794b"}],"default":{"alias":"VW Docking Station 1(31)","value":"5dc253a295a03d00127c794b"},"properties":[],"key":"station"}],"func_name":"dock"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},{"alias":"intersection test area","value":"5dbfa93395a03d00127c78d3"},{"alias":"maze to table 2","value":"5dc3a8e795a03d00127c7959"}],"default":{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},"properties":[],"key":"resource","options":{"user_input":false}}],"func_name":"lock"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},{"alias":"In Front Of Cleaning Robot Team","value":"5cd8f059ab582e775cd6ca62"},{"alias":"In Front Of Military Robot Team","value":"5cd8f0a8ab582e775cd6ca63"},{"alias":"In Front Of QA Team","value":"5cd8f0b4ab582e775cd6ca64"},{"alias":"In Front Of Information desk","value":"5cd8f0ccab582e775cd6ca65"},{"alias":"R&D Center Entrance","value":"5cd8f10eab582e775cd6ca68"},{"alias":"Table 1","value":"5cd8f1b9ab582e775cd6ca6d"},{"alias":"Table 2","value":"5cd8f1c2ab582e775cd6ca6e"},{"alias":"Table 3","value":"5cd8f1c8ab582e775cd6ca6f"},{"alias":"Head Office Entrance","value":"5cd8f21aab582e775cd6ca72"},{"alias":"In Front of Meeting Room","value":"5cd8f22fab582e775cd6ca73"},{"alias":"W_E4_1","value":"5cf4f9c8f057bd4671f4d6a5"},{"alias":"W_E4_2","value":"5cf4f9e2f057bd4671f4d6a6"},{"alias":"W_E3_1","value":"5cf4fa25f057bd4671f4d6a7"},{"alias":"W_E3_2","value":"5cf4fa30f057bd4671f4d6a8"},{"alias":"W_E1_1","value":"5cf4fa4ef057bd4671f4d6a9"},{"alias":"W_E1_2","value":"5cf4fa5ef057bd4671f4d6aa"},{"alias":"W_X3_1","value":"5d01a8d3a25bd45942161731"},{"alias":"W_X3_2","value":"5d01a8fda25bd45942161732"},{"alias":"W_X3_3","value":"5d01a923a25bd45942161733"},{"alias":"W_X3_4","value":"5d01a930a25bd45942161734"},{"alias":"3F hall","value":"5d09969e13197f78beb339e8"},{"alias":"W_A3_1","value":"5d099d6413197f78beb339e9"},{"alias":"W_A3_2","value":"5d099d8313197f78beb339ea"},{"alias":"W_A3_3","value":"5d099da113197f78beb339eb"},{"alias":"W_A3_4","value":"5d099dac13197f78beb339ec"},{"alias":"Cafe Home","value":"5d3154df7a86d200120faf05"},{"alias":"Cafe Docking Front","value":"5d3154f97a86d200120faf06"},{"alias":"inside intersection","value":"5dbfa95295a03d00127c78d4"},{"alias":"intersection A waiting 1","value":"5dbfa97795a03d00127c78d5"},{"alias":"intersection B waiting 1","value":"5dbfa98d95a03d00127c78d6"},{"alias":"intersection C waiting 1","value":"5dbfb07895a03d00127c78d8"},{"alias":"intersection B side","value":"5dbfb10295a03d00127c78da"},{"alias":"intersection C side","value":"5dbfb11095a03d00127c78db"},{"alias":"intersection A side","value":"5dbfb54195a03d00127c78dc"},{"alias":"W_X3_5","value":"5dc0cbe395a03d00127c78df"},{"alias":"W_X3_6","value":"5dc0cbf295a03d00127c78e0"},{"alias":"intersection A waiting 2","value":"5dc2208895a03d00127c7940"},{"alias":"intersection A waiting 3","value":"5dc220f195a03d00127c7941"},{"alias":"intersection B waiting 2","value":"5dc2212f95a03d00127c7942"},{"alias":"intersection B waiting 3","value":"5dc2215b95a03d00127c7943"},{"alias":"intersection C waiting 2","value":"5dc221a895a03d00127c7944"},{"alias":"intersection C waiting 3","value":"5dc221b995a03d00127c7945"},{"alias":"intersection D side","value":"5dc221cc95a03d00127c7946"},{"alias":"intersection D waiting 1","value":"5dc221e095a03d00127c7947"},{"alias":"intersection D waiting 2","value":"5dc221f095a03d00127c7948"},{"alias":"intersection D waiting 3","value":"5dc2220b95a03d00127c7949"},{"alias":"3F Docking Front 1","value":"5dc2580f95a03d00127c794c"},{"alias":"3F Docking Front2","value":"5dc2582195a03d00127c794d"},{"alias":"3F Docking Front 3","value":"5dc2583795a03d00127c794e"},{"alias":"W_A3_5","value":"5dc3a69e95a03d00127c794f"},{"alias":"W_A3_6","value":"5dc3a6f195a03d00127c7950"},{"alias":"W_E3_3","value":"5dc3a72495a03d00127c7951"},{"alias":"W_E1_3","value":"5dc3a75095a03d00127c7952"},{"alias":"MAZE_W1","value":"5dc3a91295a03d00127c795a"},{"alias":"MAZE_W2","value":"5dc3a92295a03d00127c795b"},{"alias":"MAZE_W3","value":"5dc3a93195a03d00127c795c"}],"default":{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},"properties":[],"key":"destination","options":{"min":0,"user_input":false,"max":0}},{"type":"boolean","format":null,"value":null,"domain":[{"alias":"True","value":true},{"alias":"False","value":false}],"default":{"alias":"True","value":true},"properties":[],"key":"goal_finishing"},{"type":"number","format":null,"value":null,"domain":[0,10,20,30,40,50,60],"default":30,"properties":[],"key":"finishing_timeout","options":{"user_input":true,"min":0,"max":300}}],"func_name":"move"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[],"default":null,"properties":[],"key":"default_sender","name":"Default Sender","options":{"user_input":true}},{"type":"string","format":null,"value":null,"domain":[],"default":null,"properties":[],"key":"default_message","name":"Default Message","options":{"user_input":true}},{"type":"object","format":null,"value":null,"domain":[],"default":null,"properties":[{"key":"email","name":"E-mail","type":"boolean","default":true},{"key":"sms","name":"SMS","type":"boolean","default":false},{"key":"web_push","name":"Web_push","type":"boolean","default":false}],"key":"default_method","name":"Default Method"},{"type":"object","format":null,"value":null,"domain":[],"default":null,"properties":[{"key":"custom_sender","name":"Custom Sender","domain":[],"type":"string","default":null,"options":{"user_input":true}},{"key":"custom_message","name":"Custom Message","domain":[],"type":"string","default":null,"options":{"user_input":true}},{"key":"custom_method","name":"Custom Method","type":"object","properties":[{"key":"email","name":"E-mail","type":"boolean","default":true},{"key":"sms","name":"SMS","type":"boolean","default":false},{"key":"web_push","name":"Web_push","type":"boolean","default":false}]},{"key":"receiver_id","name":"Receiver ID","domain":[],"type":"string","default":null,"options":{"user_input":true}},{"key":"receiver_type","name":"Receiver Type","domain":[],"type":"string","default":null,"options":{"user_input":true}}],"key":"receivers","name":"Receiver"}],"func_name":"notify"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},{"alias":"intersection test area","value":"5dbfa93395a03d00127c78d3"},{"alias":"maze to table 2","value":"5dc3a8e795a03d00127c7959"}],"default":{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},"properties":[],"key":"resource","options":{"user_input":false}},{"type":"object","format":null,"value":null,"domain":[],"default":null,"properties":[],"key":"quit","options":{"user_input":false}}],"func_name":"unlock"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},{"alias":"In Front Of Cleaning Robot Team","value":"5cd8f059ab582e775cd6ca62"},{"alias":"In Front Of Military Robot Team","value":"5cd8f0a8ab582e775cd6ca63"},{"alias":"In Front Of QA Team","value":"5cd8f0b4ab582e775cd6ca64"},{"alias":"In Front Of Information desk","value":"5cd8f0ccab582e775cd6ca65"},{"alias":"R&D Center Entrance","value":"5cd8f10eab582e775cd6ca68"},{"alias":"Table 1","value":"5cd8f1b9ab582e775cd6ca6d"},{"alias":"Table 2","value":"5cd8f1c2ab582e775cd6ca6e"},{"alias":"Table 3","value":"5cd8f1c8ab582e775cd6ca6f"},{"alias":"Head Office Entrance","value":"5cd8f21aab582e775cd6ca72"},{"alias":"In Front of Meeting Room","value":"5cd8f22fab582e775cd6ca73"},{"alias":"W_E4_1","value":"5cf4f9c8f057bd4671f4d6a5"},{"alias":"W_E4_2","value":"5cf4f9e2f057bd4671f4d6a6"},{"alias":"W_E3_1","value":"5cf4fa25f057bd4671f4d6a7"},{"alias":"W_E3_2","value":"5cf4fa30f057bd4671f4d6a8"},{"alias":"W_E1_1","value":"5cf4fa4ef057bd4671f4d6a9"},{"alias":"W_E1_2","value":"5cf4fa5ef057bd4671f4d6aa"},{"alias":"W_X3_1","value":"5d01a8d3a25bd45942161731"},{"alias":"W_X3_2","value":"5d01a8fda25bd45942161732"},{"alias":"W_X3_3","value":"5d01a923a25bd45942161733"},{"alias":"W_X3_4","value":"5d01a930a25bd45942161734"},{"alias":"3F hall","value":"5d09969e13197f78beb339e8"},{"alias":"W_A3_1","value":"5d099d6413197f78beb339e9"},{"alias":"W_A3_2","value":"5d099d8313197f78beb339ea"},{"alias":"W_A3_3","value":"5d099da113197f78beb339eb"},{"alias":"W_A3_4","value":"5d099dac13197f78beb339ec"},{"alias":"Cafe Home","value":"5d3154df7a86d200120faf05"},{"alias":"Cafe Docking Front","value":"5d3154f97a86d200120faf06"},{"alias":"inside intersection","value":"5dbfa95295a03d00127c78d4"},{"alias":"intersection A waiting 1","value":"5dbfa97795a03d00127c78d5"},{"alias":"intersection B waiting 1","value":"5dbfa98d95a03d00127c78d6"},{"alias":"intersection C waiting 1","value":"5dbfb07895a03d00127c78d8"},{"alias":"intersection B side","value":"5dbfb10295a03d00127c78da"},{"alias":"intersection C side","value":"5dbfb11095a03d00127c78db"},{"alias":"intersection A side","value":"5dbfb54195a03d00127c78dc"},{"alias":"W_X3_5","value":"5dc0cbe395a03d00127c78df"},{"alias":"W_X3_6","value":"5dc0cbf295a03d00127c78e0"},{"alias":"intersection A waiting 2","value":"5dc2208895a03d00127c7940"},{"alias":"intersection A waiting 3","value":"5dc220f195a03d00127c7941"},{"alias":"intersection B waiting 2","value":"5dc2212f95a03d00127c7942"},{"alias":"intersection B waiting 3","value":"5dc2215b95a03d00127c7943"},{"alias":"intersection C waiting 2","value":"5dc221a895a03d00127c7944"},{"alias":"intersection C waiting 3","value":"5dc221b995a03d00127c7945"},{"alias":"intersection D side","value":"5dc221cc95a03d00127c7946"},{"alias":"intersection D waiting 1","value":"5dc221e095a03d00127c7947"},{"alias":"intersection D waiting 2","value":"5dc221f095a03d00127c7948"},{"alias":"intersection D waiting 3","value":"5dc2220b95a03d00127c7949"},{"alias":"3F Docking Front 1","value":"5dc2580f95a03d00127c794c"},{"alias":"3F Docking Front2","value":"5dc2582195a03d00127c794d"},{"alias":"3F Docking Front 3","value":"5dc2583795a03d00127c794e"},{"alias":"W_A3_5","value":"5dc3a69e95a03d00127c794f"},{"alias":"W_A3_6","value":"5dc3a6f195a03d00127c7950"},{"alias":"W_E3_3","value":"5dc3a72495a03d00127c7951"},{"alias":"W_E1_3","value":"5dc3a75095a03d00127c7952"},{"alias":"MAZE_W1","value":"5dc3a91295a03d00127c795a"},{"alias":"MAZE_W2","value":"5dc3a92295a03d00127c795b"},{"alias":"MAZE_W3","value":"5dc3a93195a03d00127c795c"},{"alias":"1f_prohibited","value":"5cf4f688f057bd4671f4d69d"},{"alias":"1f_prohibited","value":"5cf4f6aff057bd4671f4d69e"},{"alias":"1f_prohibited","value":"5cf4f6dbf057bd4671f4d69f"},{"alias":"1f_prohibited","value":"5cf4f6f9f057bd4671f4d6a0"},{"alias":"1f_prohibited","value":"5cf4f70bf057bd4671f4d6a1"},{"alias":"1f_prohibited","value":"5cf4f736f057bd4671f4d6a2"},{"alias":"3f_prohibited","value":"5cf4f7c5f057bd4671f4d6a3"},{"alias":"3f_prohibited","value":"5cf4f7f7f057bd4671f4d6a4"},{"alias":"1f_prohibited","value":"5cf4fa95f057bd4671f4d6ab"},{"alias":"4f_zone_prohibit_backdoor","value":"5dc3a78895a03d00127c7953"},{"alias":"1f_zone_prohibie_maze1","value":"5dc3a7e395a03d00127c7954"},{"alias":"1f_zone_prohibie_maze2","value":"5dc3a80595a03d00127c7955"},{"alias":"1f_zone_prohibie_maze3","value":"5dc3a83295a03d00127c7956"},{"alias":"1f_zone_prohibie_maze4","value":"5dc3a85595a03d00127c7957"},{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},{"alias":"intersection test area","value":"5dbfa93395a03d00127c78d3"},{"alias":"maze to table 2","value":"5dc3a8e795a03d00127c7959"},{"alias":"R&D Autodoor","value":"5cd8f281ab582e775cd6ca81"},{"alias":"VW Docking Station 2","value":"5cd8f0e5ab582e775cd6ca66"},{"alias":"VW Docking Station 3","value":"5cd8f0f7ab582e775cd6ca67"},{"alias":"Cafe Robot Docking Station","value":"5cd8f1b0ab582e775cd6ca6c"},{"alias":"VW Docking Station 1","value":"5dc253a295a03d00127c794b"},{"alias":"Center Elevator","value":"5cd8f246ab582e775cd6ca74"},{"alias":"Center Elevator 1F","value":"5cd8f256ab582e775cd6ca75"},{"alias":"Center Elevator 3F","value":"5cd8f266ab582e775cd6ca76"},{"alias":"Center Elevator 4F","value":"5cd8f271ab582e775cd6ca77"},{"alias":"test_pro","value":"5e45fe57479c5a0019825585"}],"default":{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},"properties":[],"key":"resource_id"},{"type":"string","format":null,"value":null,"domain":[{"alias":"ExclusiveArea","value":"ExclusiveArea"},{"alias":"Autodoor","value":"Autodoor"},{"alias":"Elevator","value":"Teleporter"}],"default":{"alias":"ExclusiveArea","value":"ExclusiveArea"},"properties":[],"key":"resource_type"},{"type":"object","format":null,"value":null,"domain":[{}],"default":null,"properties":[],"key":"params"}],"func_name":"use_resource"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"Duration","value":"duration"},{"alias":"Controlled by system","value":"signal"},{"alias":"Until button press","value":"human_input"}],"default":"human_input","properties":[],"key":"type","options":{"user_input":false,"regex":null}},{"type":"number","format":null,"value":null,"domain":[{"alias":"forever","value":"-1"},{"alias":"10000 ms","value":"10000"},{"alias":"20000 ms","value":"20000"},{"alias":"30000 ms","value":"30000"},{"alias":"40000 ms","value":"40000"},{"alias":"50000 ms","value":"50000"},{"alias":"60000 ms","value":"60000"}],"default":30000,"properties":[],"key":"timeout","options":{"user_input":true,"regex":null}},{"type":"string","format":null,"value":null,"domain":["revision","success","fail"],"default":"revision","properties":[],"key":"param"},{"type":"boolean","format":null,"value":null,"domain":[{"alias":"True","value":true},{"alias":"False","value":false}],"default":{"alias":"False","value":false},"properties":[],"key":"door_behavior"}],"func_name":"wait"}],"created_at":"2020-02-20T04:58:30.227Z","description":"mocking GoCart Delivery in virtual environment","updated_at":"2020-02-25T00:43:45.123Z","id":"5e4e11f696d5900e0f298a96"},"bootstrap":true,"worker_update":{"updated_at":"2020-02-25T00:43:57.190Z"},"status":"idle","task_updated":1582591425096},"treeMemory":{"configs":{"hostname":{"concert":"localhost:10602","siteConfig":"localhost:10604"},"server":{"port":10999},"logger":{"console_level":"info","console_json":true,"file_level":"info"},"assets":{"base":"./assets","map":"/maps"},"vw_archives":"./vw_archives","temp":{"iot_operator":"localhost:10702"},"action":{"move":"direct"},"taskName":"Virtual Delivery","taskDescription":"mocking GoCart Delivery in virtual environment"},"uuid":"a1","actionDefinePaths":"/home/blcho/dev/virtual_core/src/core/VirtualWorker/actions","messageDefinePaths":"/home/blcho/dev/virtual_core/src/core/VirtualWorker/messageHandlers","tickInterval":1000,"updateInterval":2000,"activeNodes":[{"id":"e34f02c2-653b-45e7-9a13-bf83df1c29d9","category":"composite","type":"Sequence","name":"rootTree","description":"","properties":{},"children":[{"id":"67fde38e-a12b-4af0-836b-304e746aa5a1","category":"decorator","type":"UntilSuccess","name":"Until Success","description":"","properties":{"maxLoop":-1},"child":{"id":"9cf3262b-8bbe-4abb-9720-b32579334ed2","category":"composite","type":"Priority","name":"bootstrap","description":"","properties":{},"children":[{"id":"5656c1b7-7564-4237-b10e-4d63e2a630c3","category":"task","type":"Task","name":"isBooted","description":"","properties":{}},{"id":"d26af1d1-8db2-4eab-9985-3537d3dc6a07","category":"composite","type":"Sequence","name":"bootstraping","description":"","properties":{},"children":[{"id":"7fbb96fd-0a9d-48fb-8e94-23cd9366dfc0","category":"task","type":"AsyncTask","name":"initWorker","description":"","properties":{}},{"id":"d7bdf450-fcaa-4c92-ba43-968adf673e30","category":"task","type":"AsyncTask","name":"initTask","description":"","properties":{}},{"id":"d53dfa94-cbc7-470f-93c6-aa70b657e52a","category":"task","type":"AsyncTask","name":"bootCheck","description":"","properties":{}}]}]},"maxLoop":-1},{"id":"d92be6bf-04f9-421f-bae6-2a6796e19cea","category":"decorator","type":"Repeater","name":"Repeat <maxLoop>x","description":"","properties":{"maxLoop":-1},"child":{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},"maxLoop":-1}]},{"id":"d92be6bf-04f9-421f-bae6-2a6796e19cea","category":"decorator","type":"Repeater","name":"Repeat <maxLoop>x","description":"","properties":{"maxLoop":-1},"child":{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},"maxLoop":-1},{"id":"6d4df051-e6b4-4799-9eaf-5af8f9be45ef","category":"composite","type":"Sequence","name":"activated","description":"","properties":{},"children":[{"id":"68db8889-b3fc-4c64-a3c3-43952e66889d","category":"composite","type":"Sequence","name":"heartBeat","description":"","properties":{},"children":[{"id":"2687bf8e-078a-45b7-9819-12683d940516","category":"task","type":"Task","name":"updateWorker","description":"","properties":{}},{"id":"ad061e25-5528-4435-9304-363e3e14a475","category":"task","type":"Task","name":"updateTask","description":"","properties":{}}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"68569def-7ddf-4c95-8ae0-f67ff4b13015","category":"composite","type":"Sequence","name":"byStatus","description":"","properties":{},"children":[{"id":"6a48bef5-fd84-4312-8493-55c0898f9723","category":"composite","type":"Sequence","name":"idleStatus","description":"","properties":{},"children":[{"id":"0e43c429-f2a9-4056-8ae3-af9a7cac48cf","category":"task","type":"Task","name":"isIdleStatus","description":"","properties":{}},{"id":"ca6851c6-3724-4df1-ae62-a42b0df14490","category":"task","type":"Task","name":"healthCheck","description":"","properties":{}},{"id":"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5","category":"composite","type":"Priority","name":"initReport","description":"","properties":{},"children":[{"id":"e34d6571-662b-4595-98ee-3422605976ec","category":"task","type":"AsyncTask","name":"initReportFromExistReport","description":"","properties":{}},{"id":"2568af36-d161-42d5-85b7-df69795da934","category":"composite","type":"Sequence","name":"fetchNewReport","description":"","properties":{},"children":[{"id":"c6029fe7-0c27-4a54-b2b9-d7c3a2571002","category":"task","type":"AsyncTask","name":"requestRecommendation","description":"","properties":{}},{"id":"ce2630fd-bdc4-4b7e-af39-baeffae1ae32","category":"task","type":"AsyncTask","name":"getOwnership","description":"","properties":{}}]}]},{"id":"43ac1326-08b9-402d-9c53-3a389ac60795","category":"task","type":"AsyncTask","name":"handleFirstRevision","description":"","properties":{}},{"id":"7d203e78-650b-43fa-b683-d87f64e712dc","category":"task","type":"AsyncTask","name":"startReport","description":"","properties":{}}]},{"id":"d2f19b35-5ad8-4cec-a525-cb698b6f98ad","category":"composite","type":"Sequence","name":"busyStatus","description":"","properties":{},"children":[{"id":"f4f87b0e-7e84-44e3-aff3-e1ce9d6f9fea","category":"task","type":"Task","name":"isBusyStatus","description":"","properties":{}},{"id":"0edb226e-8a4d-4dfc-9c7e-7ec67ebda281","category":"task","type":"AsyncTask","name":"validateReport","description":"","properties":{}},{"id":"846e9151-e5d7-49bb-934e-9a8bf5d105c9","category":"decorator","type":"RewindWhenRunning","name":"RewindWhenRunning","description":"","properties":{},"child":{"id":"f5c21a94-5315-4875-8c4f-2940576352c0","category":"composite","type":"Sequence","name":"checkRevisionThenKeepProcessing","description":"","properties":{},"children":[{"id":"5001d10a-ad65-43e4-9642-ecd07beccd60","category":"task","type":"AsyncTask","name":"checkRevision","description":"","properties":{}},{"id":"a7df776b-ce8b-4bad-91b4-c0a7829b8f7d","category":"task","type":"AsyncTask","name":"processInstruction","description":"","properties":{}}]}},{"id":"c63fba35-10ff-4083-9509-061513c8b125","category":"task","type":"AsyncTask","name":"finishReport","description":"","properties":{}}]},{"id":"4d8f410a-a5d1-40d0-949a-348cdb2caaf3","category":"composite","type":"Sequence","name":"offlineStatus","description":"","properties":{},"children":[{"id":"6911949f-2479-4a48-aa90-1ed1c108bcdd","category":"task","type":"Task","name":"isOfflineStatus","description":"","properties":{}}]},{"id":"ca2f9b9a-c495-4f71-9f5d-2e853df35ff4","category":"composite","type":"Sequence","name":"errorStatus","description":"","properties":{},"children":[{"id":"ab56e764-5c26-490c-9fe2-8c942cc2a5c4","category":"task","type":"Task","name":"isErrorStatus","description":"","properties":{}}]}]}]},{"id":"2aaf2074-720f-4986-9cfb-337b812ba3f2","category":"composite","type":"Priority","name":"externalControl","description":"","properties":{},"children":[{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}]},{"id":"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7","category":"task","type":"AsyncTask","name":"handleControlMessage","description":"","properties":{}}],"worker":{"capability":{"tasks":[]},"type":"unknown worker type","release_version":"0.0.0","type_specific":{"battery":{"battery_level":75,"charging_status":0},"location":{"map":"5cd8ef9d65b9b770ac03bd1d","pose2d":{"x":13.68,"y":11.89,"theta":1.05},"semantic_location":null}},"_id":"5e5338bd1f90480013ec26f1","uuid":"a1","__v":0,"created_at":"2020-02-24T02:45:17.790Z","name":"VirtualWorker(a1)","updated_at":"2020-02-25T00:43:53.129Z","status":"offline","id":"5e5338bd1f90480013ec26f1"},"task":{"_id":"5e4e11f696d5900e0f298a96","actions_hash":"33143578c290ef33b280aa5a61048f5d5915c35b4c809edf709c5485ea314513","name":"Virtual Delivery","__v":0,"actions":[{"fixed":false,"args":[{"type":"number","format":null,"value":null,"domain":[{"alias":"VW Docking Station 2(32)","value":"5cd8f0e5ab582e775cd6ca66"},{"alias":"VW Docking Station 3(33)","value":"5cd8f0f7ab582e775cd6ca67"},{"alias":"Cafe Robot Docking Station(11)","value":"5cd8f1b0ab582e775cd6ca6c"},{"alias":"VW Docking Station 1(31)","value":"5dc253a295a03d00127c794b"}],"default":{"alias":"VW Docking Station 1(31)","value":"5dc253a295a03d00127c794b"},"properties":[],"key":"station"}],"func_name":"dock"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},{"alias":"intersection test area","value":"5dbfa93395a03d00127c78d3"},{"alias":"maze to table 2","value":"5dc3a8e795a03d00127c7959"}],"default":{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},"properties":[],"key":"resource","options":{"user_input":false}}],"func_name":"lock"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},{"alias":"In Front Of Cleaning Robot Team","value":"5cd8f059ab582e775cd6ca62"},{"alias":"In Front Of Military Robot Team","value":"5cd8f0a8ab582e775cd6ca63"},{"alias":"In Front Of QA Team","value":"5cd8f0b4ab582e775cd6ca64"},{"alias":"In Front Of Information desk","value":"5cd8f0ccab582e775cd6ca65"},{"alias":"R&D Center Entrance","value":"5cd8f10eab582e775cd6ca68"},{"alias":"Table 1","value":"5cd8f1b9ab582e775cd6ca6d"},{"alias":"Table 2","value":"5cd8f1c2ab582e775cd6ca6e"},{"alias":"Table 3","value":"5cd8f1c8ab582e775cd6ca6f"},{"alias":"Head Office Entrance","value":"5cd8f21aab582e775cd6ca72"},{"alias":"In Front of Meeting Room","value":"5cd8f22fab582e775cd6ca73"},{"alias":"W_E4_1","value":"5cf4f9c8f057bd4671f4d6a5"},{"alias":"W_E4_2","value":"5cf4f9e2f057bd4671f4d6a6"},{"alias":"W_E3_1","value":"5cf4fa25f057bd4671f4d6a7"},{"alias":"W_E3_2","value":"5cf4fa30f057bd4671f4d6a8"},{"alias":"W_E1_1","value":"5cf4fa4ef057bd4671f4d6a9"},{"alias":"W_E1_2","value":"5cf4fa5ef057bd4671f4d6aa"},{"alias":"W_X3_1","value":"5d01a8d3a25bd45942161731"},{"alias":"W_X3_2","value":"5d01a8fda25bd45942161732"},{"alias":"W_X3_3","value":"5d01a923a25bd45942161733"},{"alias":"W_X3_4","value":"5d01a930a25bd45942161734"},{"alias":"3F hall","value":"5d09969e13197f78beb339e8"},{"alias":"W_A3_1","value":"5d099d6413197f78beb339e9"},{"alias":"W_A3_2","value":"5d099d8313197f78beb339ea"},{"alias":"W_A3_3","value":"5d099da113197f78beb339eb"},{"alias":"W_A3_4","value":"5d099dac13197f78beb339ec"},{"alias":"Cafe Home","value":"5d3154df7a86d200120faf05"},{"alias":"Cafe Docking Front","value":"5d3154f97a86d200120faf06"},{"alias":"inside intersection","value":"5dbfa95295a03d00127c78d4"},{"alias":"intersection A waiting 1","value":"5dbfa97795a03d00127c78d5"},{"alias":"intersection B waiting 1","value":"5dbfa98d95a03d00127c78d6"},{"alias":"intersection C waiting 1","value":"5dbfb07895a03d00127c78d8"},{"alias":"intersection B side","value":"5dbfb10295a03d00127c78da"},{"alias":"intersection C side","value":"5dbfb11095a03d00127c78db"},{"alias":"intersection A side","value":"5dbfb54195a03d00127c78dc"},{"alias":"W_X3_5","value":"5dc0cbe395a03d00127c78df"},{"alias":"W_X3_6","value":"5dc0cbf295a03d00127c78e0"},{"alias":"intersection A waiting 2","value":"5dc2208895a03d00127c7940"},{"alias":"intersection A waiting 3","value":"5dc220f195a03d00127c7941"},{"alias":"intersection B waiting 2","value":"5dc2212f95a03d00127c7942"},{"alias":"intersection B waiting 3","value":"5dc2215b95a03d00127c7943"},{"alias":"intersection C waiting 2","value":"5dc221a895a03d00127c7944"},{"alias":"intersection C waiting 3","value":"5dc221b995a03d00127c7945"},{"alias":"intersection D side","value":"5dc221cc95a03d00127c7946"},{"alias":"intersection D waiting 1","value":"5dc221e095a03d00127c7947"},{"alias":"intersection D waiting 2","value":"5dc221f095a03d00127c7948"},{"alias":"intersection D waiting 3","value":"5dc2220b95a03d00127c7949"},{"alias":"3F Docking Front 1","value":"5dc2580f95a03d00127c794c"},{"alias":"3F Docking Front2","value":"5dc2582195a03d00127c794d"},{"alias":"3F Docking Front 3","value":"5dc2583795a03d00127c794e"},{"alias":"W_A3_5","value":"5dc3a69e95a03d00127c794f"},{"alias":"W_A3_6","value":"5dc3a6f195a03d00127c7950"},{"alias":"W_E3_3","value":"5dc3a72495a03d00127c7951"},{"alias":"W_E1_3","value":"5dc3a75095a03d00127c7952"},{"alias":"MAZE_W1","value":"5dc3a91295a03d00127c795a"},{"alias":"MAZE_W2","value":"5dc3a92295a03d00127c795b"},{"alias":"MAZE_W3","value":"5dc3a93195a03d00127c795c"}],"default":{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},"properties":[],"key":"destination","options":{"min":0,"user_input":false,"max":0}},{"type":"boolean","format":null,"value":null,"domain":[{"alias":"True","value":true},{"alias":"False","value":false}],"default":{"alias":"True","value":true},"properties":[],"key":"goal_finishing"},{"type":"number","format":null,"value":null,"domain":[0,10,20,30,40,50,60],"default":30,"properties":[],"key":"finishing_timeout","options":{"user_input":true,"min":0,"max":300}}],"func_name":"move"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[],"default":null,"properties":[],"key":"default_sender","name":"Default Sender","options":{"user_input":true}},{"type":"string","format":null,"value":null,"domain":[],"default":null,"properties":[],"key":"default_message","name":"Default Message","options":{"user_input":true}},{"type":"object","format":null,"value":null,"domain":[],"default":null,"properties":[{"key":"email","name":"E-mail","type":"boolean","default":true},{"key":"sms","name":"SMS","type":"boolean","default":false},{"key":"web_push","name":"Web_push","type":"boolean","default":false}],"key":"default_method","name":"Default Method"},{"type":"object","format":null,"value":null,"domain":[],"default":null,"properties":[{"key":"custom_sender","name":"Custom Sender","domain":[],"type":"string","default":null,"options":{"user_input":true}},{"key":"custom_message","name":"Custom Message","domain":[],"type":"string","default":null,"options":{"user_input":true}},{"key":"custom_method","name":"Custom Method","type":"object","properties":[{"key":"email","name":"E-mail","type":"boolean","default":true},{"key":"sms","name":"SMS","type":"boolean","default":false},{"key":"web_push","name":"Web_push","type":"boolean","default":false}]},{"key":"receiver_id","name":"Receiver ID","domain":[],"type":"string","default":null,"options":{"user_input":true}},{"key":"receiver_type","name":"Receiver Type","domain":[],"type":"string","default":null,"options":{"user_input":true}}],"key":"receivers","name":"Receiver"}],"func_name":"notify"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},{"alias":"intersection test area","value":"5dbfa93395a03d00127c78d3"},{"alias":"maze to table 2","value":"5dc3a8e795a03d00127c7959"}],"default":{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},"properties":[],"key":"resource","options":{"user_input":false}},{"type":"object","format":null,"value":null,"domain":[],"default":null,"properties":[],"key":"quit","options":{"user_input":false}}],"func_name":"unlock"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},{"alias":"In Front Of Cleaning Robot Team","value":"5cd8f059ab582e775cd6ca62"},{"alias":"In Front Of Military Robot Team","value":"5cd8f0a8ab582e775cd6ca63"},{"alias":"In Front Of QA Team","value":"5cd8f0b4ab582e775cd6ca64"},{"alias":"In Front Of Information desk","value":"5cd8f0ccab582e775cd6ca65"},{"alias":"R&D Center Entrance","value":"5cd8f10eab582e775cd6ca68"},{"alias":"Table 1","value":"5cd8f1b9ab582e775cd6ca6d"},{"alias":"Table 2","value":"5cd8f1c2ab582e775cd6ca6e"},{"alias":"Table 3","value":"5cd8f1c8ab582e775cd6ca6f"},{"alias":"Head Office Entrance","value":"5cd8f21aab582e775cd6ca72"},{"alias":"In Front of Meeting Room","value":"5cd8f22fab582e775cd6ca73"},{"alias":"W_E4_1","value":"5cf4f9c8f057bd4671f4d6a5"},{"alias":"W_E4_2","value":"5cf4f9e2f057bd4671f4d6a6"},{"alias":"W_E3_1","value":"5cf4fa25f057bd4671f4d6a7"},{"alias":"W_E3_2","value":"5cf4fa30f057bd4671f4d6a8"},{"alias":"W_E1_1","value":"5cf4fa4ef057bd4671f4d6a9"},{"alias":"W_E1_2","value":"5cf4fa5ef057bd4671f4d6aa"},{"alias":"W_X3_1","value":"5d01a8d3a25bd45942161731"},{"alias":"W_X3_2","value":"5d01a8fda25bd45942161732"},{"alias":"W_X3_3","value":"5d01a923a25bd45942161733"},{"alias":"W_X3_4","value":"5d01a930a25bd45942161734"},{"alias":"3F hall","value":"5d09969e13197f78beb339e8"},{"alias":"W_A3_1","value":"5d099d6413197f78beb339e9"},{"alias":"W_A3_2","value":"5d099d8313197f78beb339ea"},{"alias":"W_A3_3","value":"5d099da113197f78beb339eb"},{"alias":"W_A3_4","value":"5d099dac13197f78beb339ec"},{"alias":"Cafe Home","value":"5d3154df7a86d200120faf05"},{"alias":"Cafe Docking Front","value":"5d3154f97a86d200120faf06"},{"alias":"inside intersection","value":"5dbfa95295a03d00127c78d4"},{"alias":"intersection A waiting 1","value":"5dbfa97795a03d00127c78d5"},{"alias":"intersection B waiting 1","value":"5dbfa98d95a03d00127c78d6"},{"alias":"intersection C waiting 1","value":"5dbfb07895a03d00127c78d8"},{"alias":"intersection B side","value":"5dbfb10295a03d00127c78da"},{"alias":"intersection C side","value":"5dbfb11095a03d00127c78db"},{"alias":"intersection A side","value":"5dbfb54195a03d00127c78dc"},{"alias":"W_X3_5","value":"5dc0cbe395a03d00127c78df"},{"alias":"W_X3_6","value":"5dc0cbf295a03d00127c78e0"},{"alias":"intersection A waiting 2","value":"5dc2208895a03d00127c7940"},{"alias":"intersection A waiting 3","value":"5dc220f195a03d00127c7941"},{"alias":"intersection B waiting 2","value":"5dc2212f95a03d00127c7942"},{"alias":"intersection B waiting 3","value":"5dc2215b95a03d00127c7943"},{"alias":"intersection C waiting 2","value":"5dc221a895a03d00127c7944"},{"alias":"intersection C waiting 3","value":"5dc221b995a03d00127c7945"},{"alias":"intersection D side","value":"5dc221cc95a03d00127c7946"},{"alias":"intersection D waiting 1","value":"5dc221e095a03d00127c7947"},{"alias":"intersection D waiting 2","value":"5dc221f095a03d00127c7948"},{"alias":"intersection D waiting 3","value":"5dc2220b95a03d00127c7949"},{"alias":"3F Docking Front 1","value":"5dc2580f95a03d00127c794c"},{"alias":"3F Docking Front2","value":"5dc2582195a03d00127c794d"},{"alias":"3F Docking Front 3","value":"5dc2583795a03d00127c794e"},{"alias":"W_A3_5","value":"5dc3a69e95a03d00127c794f"},{"alias":"W_A3_6","value":"5dc3a6f195a03d00127c7950"},{"alias":"W_E3_3","value":"5dc3a72495a03d00127c7951"},{"alias":"W_E1_3","value":"5dc3a75095a03d00127c7952"},{"alias":"MAZE_W1","value":"5dc3a91295a03d00127c795a"},{"alias":"MAZE_W2","value":"5dc3a92295a03d00127c795b"},{"alias":"MAZE_W3","value":"5dc3a93195a03d00127c795c"},{"alias":"1f_prohibited","value":"5cf4f688f057bd4671f4d69d"},{"alias":"1f_prohibited","value":"5cf4f6aff057bd4671f4d69e"},{"alias":"1f_prohibited","value":"5cf4f6dbf057bd4671f4d69f"},{"alias":"1f_prohibited","value":"5cf4f6f9f057bd4671f4d6a0"},{"alias":"1f_prohibited","value":"5cf4f70bf057bd4671f4d6a1"},{"alias":"1f_prohibited","value":"5cf4f736f057bd4671f4d6a2"},{"alias":"3f_prohibited","value":"5cf4f7c5f057bd4671f4d6a3"},{"alias":"3f_prohibited","value":"5cf4f7f7f057bd4671f4d6a4"},{"alias":"1f_prohibited","value":"5cf4fa95f057bd4671f4d6ab"},{"alias":"4f_zone_prohibit_backdoor","value":"5dc3a78895a03d00127c7953"},{"alias":"1f_zone_prohibie_maze1","value":"5dc3a7e395a03d00127c7954"},{"alias":"1f_zone_prohibie_maze2","value":"5dc3a80595a03d00127c7955"},{"alias":"1f_zone_prohibie_maze3","value":"5dc3a83295a03d00127c7956"},{"alias":"1f_zone_prohibie_maze4","value":"5dc3a85595a03d00127c7957"},{"alias":"3F Exclusive Area","value":"5cd8f16eab582e775cd6ca6a"},{"alias":"intersection test area","value":"5dbfa93395a03d00127c78d3"},{"alias":"maze to table 2","value":"5dc3a8e795a03d00127c7959"},{"alias":"R&D Autodoor","value":"5cd8f281ab582e775cd6ca81"},{"alias":"VW Docking Station 2","value":"5cd8f0e5ab582e775cd6ca66"},{"alias":"VW Docking Station 3","value":"5cd8f0f7ab582e775cd6ca67"},{"alias":"Cafe Robot Docking Station","value":"5cd8f1b0ab582e775cd6ca6c"},{"alias":"VW Docking Station 1","value":"5dc253a295a03d00127c794b"},{"alias":"Center Elevator","value":"5cd8f246ab582e775cd6ca74"},{"alias":"Center Elevator 1F","value":"5cd8f256ab582e775cd6ca75"},{"alias":"Center Elevator 3F","value":"5cd8f266ab582e775cd6ca76"},{"alias":"Center Elevator 4F","value":"5cd8f271ab582e775cd6ca77"},{"alias":"test_pro","value":"5e45fe57479c5a0019825585"}],"default":{"alias":"In Front Of Gocart Team","value":"5cd8f043ab582e775cd6ca61"},"properties":[],"key":"resource_id"},{"type":"string","format":null,"value":null,"domain":[{"alias":"ExclusiveArea","value":"ExclusiveArea"},{"alias":"Autodoor","value":"Autodoor"},{"alias":"Elevator","value":"Teleporter"}],"default":{"alias":"ExclusiveArea","value":"ExclusiveArea"},"properties":[],"key":"resource_type"},{"type":"object","format":null,"value":null,"domain":[{}],"default":null,"properties":[],"key":"params"}],"func_name":"use_resource"},{"fixed":false,"args":[{"type":"string","format":null,"value":null,"domain":[{"alias":"Duration","value":"duration"},{"alias":"Controlled by system","value":"signal"},{"alias":"Until button press","value":"human_input"}],"default":"human_input","properties":[],"key":"type","options":{"user_input":false,"regex":null}},{"type":"number","format":null,"value":null,"domain":[{"alias":"forever","value":"-1"},{"alias":"10000 ms","value":"10000"},{"alias":"20000 ms","value":"20000"},{"alias":"30000 ms","value":"30000"},{"alias":"40000 ms","value":"40000"},{"alias":"50000 ms","value":"50000"},{"alias":"60000 ms","value":"60000"}],"default":30000,"properties":[],"key":"timeout","options":{"user_input":true,"regex":null}},{"type":"string","format":null,"value":null,"domain":["revision","success","fail"],"default":"revision","properties":[],"key":"param"},{"type":"boolean","format":null,"value":null,"domain":[{"alias":"True","value":true},{"alias":"False","value":false}],"default":{"alias":"False","value":false},"properties":[],"key":"door_behavior"}],"func_name":"wait"}],"created_at":"2020-02-20T04:58:30.227Z","description":"mocking GoCart Delivery in virtual environment","updated_at":"2020-02-25T00:43:45.123Z","id":"5e4e11f696d5900e0f298a96"},"bootstrap":true,"worker_update":{"updated_at":"2020-02-25T00:43:57.190Z"},"status":"idle","task_updated":1582591425096},"node":{"e34f02c2-653b-45e7-9a13-bf83df1c29d9":{"isOpen":true,"runningChild":1,"lastStatus":"RUNNING"},"67fde38e-a12b-4af0-836b-304e746aa5a1":{"isOpen":false,"i":0,"lastStatus":"SUCCESS"},"9cf3262b-8bbe-4abb-9720-b32579334ed2":{"isOpen":false,"runningChild":0,"lastStatus":"SUCCESS"},"5656c1b7-7564-4237-b10e-4d63e2a630c3":{"isOpen":false,"lastStatus":"SUCCESS"},"d26af1d1-8db2-4eab-9985-3537d3dc6a07":{"isOpen":false,"runningChild":2,"lastStatus":"SUCCESS"},"7fbb96fd-0a9d-48fb-8e94-23cd9366dfc0":{"isOpen":false,"lastStatus":"SUCCESS"},"d7bdf450-fcaa-4c92-ba43-968adf673e30":{"isOpen":false,"lastStatus":"SUCCESS"},"d53dfa94-cbc7-470f-93c6-aa70b657e52a":{"isOpen":false,"lastStatus":"SUCCESS"},"d92be6bf-04f9-421f-bae6-2a6796e19cea":{"isOpen":true,"i":0,"lastStatus":"RUNNING"},"6d4df051-e6b4-4799-9eaf-5af8f9be45ef":{"isOpen":true,"runningChild":1,"lastStatus":"RUNNING"},"68db8889-b3fc-4c64-a3c3-43952e66889d":{"isOpen":false,"runningChild":0,"lastStatus":"SUCCESS"},"2687bf8e-078a-45b7-9819-12683d940516":{"isOpen":false,"lastStatus":"SUCCESS"},"ad061e25-5528-4435-9304-363e3e14a475":{"isOpen":false,"lastStatus":"SUCCESS"},"2aaf2074-720f-4986-9cfb-337b812ba3f2":{"isOpen":true,"runningChild":0,"lastStatus":"RUNNING"},"beaedf4f-8772-4ddc-98af-bbdd34bcb3c7":{"isOpen":true,"lastStatus":"RUNNING","asyncStatus":"RUNNING"},"68569def-7ddf-4c95-8ae0-f67ff4b13015":{"isOpen":false,"runningChild":0,"lastStatus":"FAILURE"},"6a48bef5-fd84-4312-8493-55c0898f9723":{"isOpen":false,"runningChild":2,"lastStatus":"FAILURE"},"0e43c429-f2a9-4056-8ae3-af9a7cac48cf":{"isOpen":false,"lastStatus":"SUCCESS"},"ca6851c6-3724-4df1-ae62-a42b0df14490":{"isOpen":false,"lastStatus":"SUCCESS"},"b08fa919-d5bd-4e3c-9f44-9e0bf0b6eba5":{"isOpen":false,"runningChild":1,"lastStatus":"FAILURE"},"e34d6571-662b-4595-98ee-3422605976ec":{"isOpen":false,"lastStatus":"FAILURE"},"2568af36-d161-42d5-85b7-df69795da934":{"isOpen":false,"runningChild":0,"lastStatus":"FAILURE"},"c6029fe7-0c27-4a54-b2b9-d7c3a2571002":{"isOpen":false,"lastStatus":"FAILURE"}}}},"shared":{}}}'

        return json_data

    def get_working_dir(self):
        return self._working_dir